@@ -20,10 +20,6 @@
 
 package org.klomp.snark;
 
-import java.security.Provider;
-import java.security.Security;
-
-import org.klomp.snark.bencode.*;
 
 /**
  * Main snark startup class for staticly linking with gcj.
@@ -42,6 +38,7 @@ public class StaticSnark
     //Security.addProvider(gnu);
 
     // And finally call the normal starting point.
-    Snark.main(args);
+    //Snark.main(args);
+    System.err.println("unsupported");
   }
 }
