@@ -95,6 +95,7 @@ public class I2PTunnel extends EventDispatcherImpl implements Logging {
     private final long _tunnelId;
     private final Properties _clientOptions;
     private final Set<I2PSession> _sessions;
+    private final TunnelController _controller;
 
     public static final int PACKET_DELAY = 100;
 
@@ -152,6 +153,16 @@ public class I2PTunnel extends EventDispatcherImpl implements Logging {
         this(nocli_args);
     }
 
+    /**
+     *  New standard constructor in router, with back ref to tc
+     *  @param tc may be null
+     *  @throws IllegalArgumentException
+     *  @since 0.9.48
+     */
+    public I2PTunnel(TunnelController tc) {
+        this(nocli_args, null, tc);
+    }
+
     /**
      *  See usage() for options
      *  @throws IllegalArgumentException
@@ -166,10 +177,21 @@ public class I2PTunnel extends EventDispatcherImpl implements Logging {
      *  @throws IllegalArgumentException
      */
     public I2PTunnel(String[] args, ConnectionEventListener lsnr) {
+        this(args, lsnr, null);
+    }
+
+    /**
+     *  @param lsnr may be null
+     *  @param tc may be null
+     *  @throws IllegalArgumentException
+     *  @since 0.9.48
+     */
+    private I2PTunnel(String[] args, ConnectionEventListener lsnr, TunnelController tc) {
         super();
         _context = I2PAppContext.getGlobalContext(); // new I2PAppContext();
         _tunnelId = __tunnelId.incrementAndGet();
         _log = _context.logManager().getLog(I2PTunnel.class);
+        _controller = tc;
         // as of 0.8.4, include context properties
         Properties p = _context.getProperties();
         _clientOptions = p;
@@ -307,6 +329,7 @@ public class I2PTunnel extends EventDispatcherImpl implements Logging {
                 try {
                     wait();
                 } catch (InterruptedException ie) {
+                    break;
                 }
             }
         }
@@ -365,6 +388,13 @@ public class I2PTunnel extends EventDispatcherImpl implements Logging {
      */
     public Properties getClientOptions() { return _clientOptions; }
     
+    /**
+     *  TunnelController that constructed this, or null.
+     *  @return controller or null
+     *  @since 0.9.48
+     */
+    TunnelController getController() { return _controller; }
+    
     private void addtask(I2PTunnelTask tsk) {
         tsk.setTunnel(this);
         if (tsk.isOpen()) {
@@ -1400,7 +1430,8 @@ public class I2PTunnel extends EventDispatcherImpl implements Logging {
             notifyEvent("owndestResult", "ok");
         } else {
             l.log("owndest yes|no\n" +
-                  "  Specifies whether to use its own destination \n" + "  for each outgoing tunnel");
+                  "  Specifies whether to use its own destination \n" + "  for each outgoing tunnel\n" +
+                  "Current setting: " + (ownDest ? "yes" : "no (shared client)"));
             notifyEvent("owndestResult", "error");
         }
     }
@@ -1692,6 +1723,10 @@ public class I2PTunnel extends EventDispatcherImpl implements Logging {
                     _clientOptions.setProperty("inbound.nickname", "I2Ping");
                 if (!_clientOptions.containsKey("outbound.nickname"))
                     _clientOptions.setProperty("outbound.nickname", "I2Ping");
+                if (!_clientOptions.containsKey("inbound.quantity"))
+                    _clientOptions.setProperty("inbound.quantity", "1");
+                if (!_clientOptions.containsKey("outbound.quantity"))
+                    _clientOptions.setProperty("outbound.quantity", "1");
             }
             I2PTunnelClientBase task = new I2Ping(l, ownDest, this, this);
             task.startRunning();
