@@ -92,7 +92,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                                          "\r\n";
     // ESR version of Firefox, same as Tor Browser
     private static final String UA_CLEARNET = "User-Agent: " +
-                                              "Mozilla/5.0 (Windows NT 10.0; rv:68.0) Gecko/20100101 Firefox/68.0" +
+                                              "Mozilla/5.0 (Windows NT 10.0; rv:78.0) Gecko/20100101 Firefox/78.0" +
                                               "\r\n";
     // overrides
     private static final String PROP_UA_I2P = "httpclient.userAgent.i2p";
@@ -1614,9 +1614,9 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
 
     public static final String DEFAULT_JUMP_SERVERS =
             //"http://i2host.i2p/cgi-bin/i2hostjump?," +
-            "http://stats.i2p/cgi-bin/jump.cgi?a=," +
-	    "http://no.i2p/jump/," +
-	    "http://i2pjump.i2p/jump/";
+            "http://stats.i2p/cgi-bin/jump.cgi?a=";
+            //"http://no.i2p/jump/," +
+            //"http://i2pjump.i2p/jump/";
             //"http://i2jump.i2p/";
 
     /** @param host ignored */
