@@ -20,6 +20,7 @@ import net.i2p.client.I2PSession;
 import net.i2p.client.I2PSessionException;
 import net.i2p.crypto.KeyGenerator;
 import net.i2p.crypto.SigType;
+import net.i2p.data.DataHelper;
 import net.i2p.data.Destination;
 import net.i2p.data.KeyCertificate;
 import net.i2p.data.PrivateKey;
@@ -35,6 +36,7 @@ import net.i2p.util.Log;
 import net.i2p.util.RandomSource;
 import net.i2p.util.SecureFile;
 import net.i2p.util.SecureFileOutputStream;
+import net.i2p.util.SimpleTimer2;
 import net.i2p.util.SystemVersion;
 
 /**
@@ -56,6 +58,7 @@ public class TunnelController implements Logging {
     private final List<String> _messages;
     private List<I2PSession> _sessions;
     private volatile TunnelState _state;
+    private volatile SimpleTimer2.TimedEvent _pkfc;
 
     /** @since 0.9.19 */
     private enum TunnelState {
@@ -213,7 +216,7 @@ public class TunnelController implements Logging {
      *                  overwrite existing ones)
      */
     public TunnelController(Properties config, String prefix, boolean createKey) {
-        _tunnel = new I2PTunnel();
+        _tunnel = new I2PTunnel(this);
         _log = I2PAppContext.getGlobalContext().logManager().getLog(TunnelController.class);
         setConfig(config, prefix);
         _messages = new ArrayList<String>(4);
@@ -485,6 +488,12 @@ public class TunnelController implements Logging {
         }
         acquire();
         changeState(TunnelState.RUNNING);
+        if ((!isClient() || getPersistentClientKey()) && getIsOfflineKeysAnySession()) {
+            File f = getPrivateKeyFile();
+            File f2 = getAlternatePrivateKeyFile();
+            _pkfc = new PKFChecker(f, f2);
+            _pkfc.schedule(5*60*1000L);
+        }
     }
 
     private void startHttpClient() {
@@ -771,6 +780,10 @@ public class TunnelController implements Logging {
                 return;
             changeState(TunnelState.STOPPING);
         }
+        if (_pkfc != null) {
+            _pkfc.cancel();
+            _pkfc = null;
+        }
         // I2PTunnel removes the session in close(),
         // so save the sessions to pass to release() and TCG
         Collection<I2PSession> sessions = getAllSessions();
@@ -791,6 +804,10 @@ public class TunnelController implements Logging {
                 return;
             changeState(TunnelState.DESTROYING);
         }
+        if (_pkfc != null) {
+            _pkfc.cancel();
+            _pkfc = null;
+        }
         // I2PTunnel removes the session in close(),
         // so save the sessions to pass to release() and TCG
         Collection<I2PSession> sessions = getAllSessions();
@@ -800,7 +817,16 @@ public class TunnelController implements Logging {
     }
 
     public void restartTunnel() {
-        stopTunnel();
+        TunnelState oldState;
+        synchronized (this) {
+            oldState = _state;
+            if (oldState != TunnelState.STOPPED)
+                stopTunnel();
+        }
+        if (oldState != TunnelState.STOPPED) {
+            long ms = _tunnel.getContext().isRouterContext() ? 100 : 500;
+            try { Thread.sleep(ms); } catch (InterruptedException ie) {}
+        }
         startTunnel();
     }
 
@@ -852,6 +878,8 @@ public class TunnelController implements Logging {
                 }
                 if (!_config.containsKey(OPT_BUNDLE_REPLY))
                     _config.setProperty(OPT_BUNDLE_REPLY, "false");
+                if (!_config.containsKey(OPT_ENCTYPE))
+                    _config.setProperty(OPT_ENCTYPE, "4,0");
             } else if (!isClient(type)) {
                 // override UI that sets it to false
                 _config.setProperty(OPT_BUNDLE_REPLY, "true");
@@ -862,6 +890,8 @@ public class TunnelController implements Logging {
                     _config.setProperty(OPT_TAGS_SEND, "20");
                 if (!_config.containsKey(OPT_LOW_TAGS))
                     _config.setProperty(OPT_LOW_TAGS, "14");
+                if (!_config.containsKey(OPT_ENCTYPE))
+                    _config.setProperty(OPT_ENCTYPE, "4,0");
             }
             // same default logic as in EditBean.getSigType() and GeneralHelper.getSigType()
             if (!isClient(type) ||
@@ -1141,28 +1171,42 @@ public class TunnelController implements Logging {
      *  @since 0.9.17
      */
     public Destination getDestination() {
-        if (_tunnel != null) {
-            List<I2PSession> sessions = _tunnel.getSessions();
-            for (int i = 0; i < sessions.size(); i++) {
-                I2PSession session = sessions.get(i);
-                Destination dest = session.getMyDestination();
-                if (dest != null)
-                    return dest;
-            }
+        List<I2PSession> sessions = _tunnel.getSessions();
+        for (int i = 0; i < sessions.size(); i++) {
+            I2PSession session = sessions.get(i);
+            Destination dest = session.getMyDestination();
+            if (dest != null)
+                return dest;
         }
         return null;
     }
 
     /**
      *  Returns false if not running.
-     *  @return true if offline keys or not running
+     *  @return true if the primary session has offline keys
      *  @since 0.9.40
      */
     public boolean getIsOfflineKeys() {
-        if (_tunnel != null) {
-            List<I2PSession> sessions = _tunnel.getSessions();
-            if (!sessions.isEmpty())
-                return sessions.get(0).isOffline();
+        List<I2PSession> sessions = _tunnel.getSessions();
+        if (!sessions.isEmpty())
+            return sessions.get(0).isOffline();
+        return false;
+    }
+
+    /**
+     *  Returns false if not running.
+     *  @return true if ANY session or subsession has offline keys
+     *  @since 0.9.48
+     */
+    private boolean getIsOfflineKeysAnySession() {
+        List<I2PSession> sessions = _tunnel.getSessions();
+        for (I2PSession sess : sessions) {
+            if (sess.isOffline())
+                return true;
+            for (I2PSession sub : sess.getSubsessions()) {
+                if (sub.isOffline())
+                    return true;
+            }
         }
         return false;
     }
@@ -1340,4 +1384,109 @@ public class TunnelController implements Logging {
     public String toString() {
         return "TC " + getType() + ' ' + getName() + " for " + _tunnel + ' ' + _state;
     }
+
+    /**
+     * Periodically check for an updated offline-signed private key file.
+     * Log if about to expire.
+     *
+     * @since 0.9.48
+     */
+    private class PKFChecker extends SimpleTimer2.TimedEvent {
+        private final List<File> files;
+        private final List<Long> stamps;
+        private boolean wasRun;
+
+        /**
+         *  caller must schedule
+         *  @param f2 may be null
+         */
+        public PKFChecker(File f, File f2) {
+            super(SimpleTimer2.getInstance());
+            files = new ArrayList<File>(2);
+            stamps = new ArrayList<Long>(2);
+            files.add(f);
+            stamps.add(Long.valueOf(f.lastModified()));
+            if (f2 != null) {
+                files.add(f2);
+                stamps.add(Long.valueOf(f2.lastModified()));
+            }
+        }
+
+        public void timeReached() {
+            if (!getIsRunning() && !getIsStarting())
+                return;
+            List<I2PSession> sessions = _tunnel.getSessions();
+            if (getIsOfflineKeysAnySession()) {
+                I2PAppContext ctx = _tunnel.getContext();
+                long now = ctx.clock().now();
+                long delay = 2*24*60*60*1000L;
+                for (int i = 0; i < files.size(); i++) {
+                    File f = files.get(i);
+                    long stamp = stamps.get(i).longValue();
+                    if (_log.shouldDebug())
+                        _log.debug("PKFC checking: " + f + " stamp: " + DataHelper.formatTime(stamp));
+                    if (stamp <= 0)
+                        continue;
+                    if (f.lastModified() > stamp) {
+                        String msg = "Private key file " + f + " with offline signature updated, restarting tunnel";
+                        _log.logAlways(Log.WARN, msg);
+                        _tunnel.log(msg);
+                        restartTunnel();
+                        return;
+                    }
+                    if (sessions.isEmpty())
+                        continue;
+                    I2PSession sess = sessions.get(0);
+                    if (i > 0) {
+                        List<I2PSession> subs = sess.getSubsessions();
+                        if (subs.isEmpty())
+                            continue;
+                        sess = subs.get(0);
+                    }
+                    if (!sess.isOffline())
+                        continue;
+                    long exp = sess.getOfflineExpiration();
+                    long remaining = exp - now;
+                    if (remaining <= 11*60*1000) {
+                        // can't sign another LS
+                        String msg;
+                        if (remaining > 0)
+                            msg = "Offline signature in private key file " + f + " for tunnel expires " + DataHelper.formatTime(exp) + ", stopping the tunnel!";
+                        else
+                            msg = "Offline signature in private key file " + f + " for tunnel expired " + DataHelper.formatTime(exp) + ", stopping the tunnel!";
+                        _log.log(Log.CRIT, msg);
+                        _tunnel.log(msg);
+                        stopTunnel();
+                        return;
+                    }
+                    long d;
+                    if (remaining < 24*60*60*1000L) {
+                        d = Math.min(60*60*1000L, remaining - (11*60*1000));
+                    } else if (remaining < 7*24*60*60*1000L) {
+                        d = 6*60*60*1000L;
+                        if (!wasRun) {
+                            d += ctx.random().nextLong(4 * delay);
+                            wasRun = true;
+                        }
+                    } else {
+                        d = 24*60*60*1000L;
+                        if (!wasRun) {
+                            delay += ctx.random().nextLong(delay);
+                            wasRun = true;
+                        }
+                    }
+                    if (remaining < 30*24*60*60*1000L) {
+                        String msg = "Offline signature in private key file " + f + " for tunnel expires in " + DataHelper.formatDuration(remaining);
+                        _log.logAlways(Log.WARN, msg);
+                        _tunnel.log("WARNING: " + msg);
+                    }
+                    if (d < delay)
+                        delay = d;
+                }
+                if (_log.shouldDebug())
+                    _log.debug("PKFC sleeping " + DataHelper.formatDuration(delay));
+                schedule(delay);
+            }
+        }
+    }
 }
