@@ -844,12 +844,15 @@ public class TunnelConfig {
 
             // As of 0.9.18, add persistent leaseset keys if not present
             // but only if we know the sigtype
-            // TODO: Put server enctype migration here
+            // Server enctype migration here
             String dflt;
-            //if (...)
-            //    dflt = "4,0";
-            //else
+            if (TunnelController.TYPE_HTTP_SERVER.equals(_type) ||
+                TunnelController.TYPE_IRC_SERVER.equals(_type) ||
+                TunnelController.TYPE_STREAMR_SERVER.equals(_type)) {
+                dflt = "4,0";
+            } else {
                 dflt = "0";
+            }
             String senc = config.getProperty(OPT + "i2cp.leaseSetEncType", dflt);
             String slstyp = config.getProperty(OPT + "i2cp.leaseSetType", "0");
             if (senc.equals("0") && slstyp.equals("0")) {
