@@ -30,9 +30,8 @@ import java.util.concurrent.atomic.AtomicBoolean;
 
 import edu.internet2.ndt.Tcpbw100;
 
-import org.json.simple.JSONObject;
-import org.json.simple.parser.JSONParser;
-import org.json.simple.parser.ParseException;
+import org.json.simple.JsonObject;
+import org.json.simple.Jsoner;
 
 import net.i2p.I2PAppContext;
 import net.i2p.data.DataHelper;
@@ -53,7 +52,8 @@ public class MLabRunner {
     // use ndt_ssl for test over ssl
     private static final String NS_URL_SSL_SSL = "https://mlab-ns.appspot.com/ndt_ssl?format=json";
     private static final String PROP_SSL = "routerconsole.bwtest.useSSL";
-    private static final boolean DEFAULT_USE_SSL = true;
+    // SSL hangs far too often
+    private static final boolean DEFAULT_USE_SSL = false;
     private static final long NS_TIMEOUT = 20*1000;
     private final I2PAppContext _context;
     private final Log _log;
@@ -152,10 +152,9 @@ public class MLabRunner {
                                 int code = eepget.getStatusCode();
                                 if (code != 200)
                                     throw new IOException("ns fetch failed: " + code);
-                                JSONParser parser = new JSONParser();
                                 byte[] b = baos.toByteArray();
                                 String s = new String(b, "ISO-8859-1");
-                                JSONObject map = (JSONObject) parser.parse(s);
+                                JsonObject map = (JsonObject) Jsoner.deserialize(s);
                                 if (map == null) {
                                     throw new IOException("no map");
                                 }
