@@ -105,28 +105,40 @@ public class PeerHelper extends HelperBase {
             out.write("<p id=\"upnpstatus\"><b>");
             out.write(_t("Status"));
             out.write(": ");
-            out.write(_t(_context.commSystem().getStatus().toStatusString()));
+            out.write(_context.commSystem().getLocalizedStatusString());
             out.write("</b></p>");
         }
 
         SortedMap<String, Transport> transports = _context.commSystem().getTransports();
         if (_transport != null && !_transport.equals("upnp")) {
+            boolean rendered = false;
             for (Map.Entry<String, Transport> e : transports.entrySet()) {
                 String style = e.getKey();
                 Transport t = e.getValue();
                 if (style.equals("NTCP") && "ntcp".equals(_transport)) {
                     NTCPTransport nt = (NTCPTransport) t;
                     render(nt, out, urlBase, sortFlags);
+                    rendered = true;
+                    break;
                 } else if (style.equals("SSU") && "ssu".equals(_transport)) {
                     UDPTransport ut = (UDPTransport) t;
                     render(ut, out, urlBase, sortFlags);
-                } else {
+                    rendered = true;
+                    break;
+                } else if (style.equals(_transport)) {
                     // pluggable (none yet)
                     t.renderStatusHTML(out, urlBase, sortFlags);
+                    rendered = true;
+                    break;
                 }
             }
-            if (!transports.isEmpty()) {
+            if (rendered) {
                 out.write(getTransportsLegend());
+            } else {
+                out.write("<p class=\"infohelp\">" +
+                          "Transport not enabled: " +
+                          DataHelper.escapeHTML(_transport) +
+                          "</p>");
             }
         } else if (_transport == null) {
             StringBuilder buf = new StringBuilder(4*1024);
@@ -293,7 +305,7 @@ public class PeerHelper extends HelperBase {
         buf.append(". ").append(_t("Limit")).append(": ").append(nt.getMaxConnections());
         //buf.append(". ").append(_t("Timeout")).append(": ").append(DataHelper.formatDuration2(_pumper.getIdleTimeout()));
         if (isAdvanced()) {
-            buf.append(". ").append(_t("Status")).append(": ").append(_t(nt.getReachabilityStatus().toStatusString()));
+            buf.append(". ").append(_t("Status")).append(": ").append(nt.getReachabilityStatus().toLocalizedStatusString(_context));
         }
         buf.append(".</h3>\n" +
                    "<div class=\"widescroll\"><table id=\"ntcpconnections\">\n" +
@@ -328,7 +340,7 @@ public class PeerHelper extends HelperBase {
                 buf.append("<img src=\"/themes/console/images/outbound.png\" alt=\"Outbound\" title=\"").append(_t("Outbound")).append("\"/>");
             buf.append("</td><td class=\"cells peeripv6\" align=\"center\">");
             if (con.isIPv6())
-                buf.append("<span class=\"backlogged\">&#x2714;</span>");
+                buf.append("<span class=\"backlogged\">&#x2713;</span>");
             else
                 buf.append("");
             buf.append("</td><td class=\"cells peeripv6\" align=\"center\">").append(con.getVersion());
@@ -364,7 +376,7 @@ public class PeerHelper extends HelperBase {
             buf.append("</td><td class=\"cells\" align=\"center\">").append(outQueue);
             buf.append("</td><td class=\"cells\" align=\"center\">");
             if (con.isBacklogged())
-                buf.append("<span class=\"backlogged\">&#x2714;</span>");
+                buf.append("<span class=\"backlogged\">&#x2713;</span>");
             else
                 buf.append("&nbsp;");
             //long readTime = con.getReadTime();
@@ -380,7 +392,7 @@ public class PeerHelper extends HelperBase {
 
         if (!peers.isEmpty()) {
 //            buf.append("<tr> <td colspan=\"11\"><hr></td></tr>\n");
-            buf.append("<tr class=\"tablefooter\"><td colspan=\"4\" align=\"left\"><b>")
+            buf.append("<tr class=\"tablefooter\"><td colspan=\"5\" align=\"left\"><b>")
                .append(ngettext("{0} peer", "{0} peers", peers.size()));
             buf.append("</b></td><td align=\"center\" nowrap><span class=\"right\"><b>").append(formatRate(bpsRecv/1000)).append("</b></span>");
             buf.append(THINSP).append("<span class=\"left\"><b>").append(formatRate(bpsSend/1000)).append("</b></span>");
@@ -457,7 +469,7 @@ public class PeerHelper extends HelperBase {
         //buf.append(". ").append(_t("Timeout")).append(": ").append(DataHelper.formatDuration2(_expireTimeout));
         final boolean isAdvanced = isAdvanced();
         if (isAdvanced) {
-            buf.append(". ").append(_t("Status")).append(": ").append(_t(ut.getReachabilityStatus().toStatusString()));
+            buf.append(". ").append(_t("Status")).append(": ").append(ut.getReachabilityStatus().toLocalizedStatusString(_context));
         }
         buf.append(".</h3>\n");
         buf.append("<div class=\"widescroll\"><table id=\"udpconnections\">\n");
