@@ -297,7 +297,7 @@ public class SybilRenderer {
      *  @since 0.9.38
      */
     private void renderBackgroundForm(Writer out, StringBuilder buf, String nonce) throws IOException {
-        long freq = _context.getProperty(Analysis.PROP_FREQUENCY, 0L);
+        long freq = _context.getProperty(Analysis.PROP_FREQUENCY, Analysis.DEFAULT_FREQUENCY);
         buf.append("<form action=\"netdb\" method=\"POST\">\n" +
                    "<input type=\"hidden\" name=\"f\" value=\"3\">\n" +
                    "<input type=\"hidden\" name=\"m\" value=\"15\">\n" +
@@ -317,10 +317,10 @@ public class SybilRenderer {
                 buf.append(_t("Never"));
             buf.append("</option>\n");
         }
-        boolean auto = _context.getBooleanProperty(Analysis.PROP_BLOCK);
+        boolean auto = _context.getProperty(Analysis.PROP_BLOCK, Analysis.DEFAULT_BLOCK);
         boolean nonff = _context.getBooleanProperty(Analysis.PROP_NONFF);
-        String thresh = _context.getProperty(Analysis.PROP_THRESHOLD, "50");
-        long days = _context.getProperty(Analysis.PROP_BLOCKTIME, 7*24*60*60*1000L) / (24*60*60*1000L);
+        String thresh = _context.getProperty(Analysis.PROP_THRESHOLD, Double.toString(Analysis.DEFAULT_BLOCK_THRESHOLD));
+        long days = _context.getProperty(Analysis.PROP_BLOCKTIME, Analysis.DEFAULT_BLOCK_TIME) / (24*60*60*1000L);
         buf.append("</select></td></tr>\n<tr><td>" +
                    "Auto-block routers?</td><td><input type=\"checkbox\" class=\"optbox\" value=\"1\" name=\"block\" ");
         if (auto)
@@ -333,7 +333,7 @@ public class SybilRenderer {
                    "Minimum threat points to block:</td><td><input type=\"text\" name=\"threshold\" value=\"").append(thresh).append("\"></td></tr>\n<tr><td>" +
                    "Days to block:</td><td><input type=\"text\" name=\"days\" value=\"").append(days).append("\"></td></tr>\n<tr><td>" +
                    "Delete stored analysis older than:</td><td><select name=\"deleteAge\">");
-        long age = _context.getProperty(Analysis.PROP_REMOVETIME, 0L);
+        long age = _context.getProperty(Analysis.PROP_REMOVETIME, Analysis.DEFAULT_REMOVE_TIME);
         for (int i = 0; i <DAYS.length; i++) {
             buf.append("<option value=\"");
             buf.append(DAYS[i]);
@@ -846,7 +846,7 @@ public class SybilRenderer {
             }
         }
         buf.append("<p><b>").append(_t("Version")).append(":</b> ").append(DataHelper.stripHTML(info.getVersion())).append("</p>\n" +
-                   "<p><b>").append(_t("Caps")).append(":</b> ").append(DataHelper.stripHTML(info.getCapabilities())).append("</p>\n");
+                   "<p><b>").append("Caps").append(":</b> ").append(DataHelper.stripHTML(info.getCapabilities())).append("</p>\n");
         String kr = info.getOption("netdb.knownRouters");
 ;
         if (kr != null) {
