@@ -14,8 +14,9 @@ package net.i2p.sam;
  *
  * @author human
  */
-public class SAMInvalidDirectionException extends Exception {
-
+class SAMInvalidDirectionException extends Exception {
+    private static final long serialVersionUID = 1;
+	
     public SAMInvalidDirectionException() {
 	super();
     }
