@@ -13,6 +13,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InterruptedIOException;
 import java.io.OutputStream;
+import java.nio.Buffer;
 import java.nio.ByteBuffer;
 import java.nio.channels.Channels;
 import java.net.ConnectException;
@@ -63,7 +64,7 @@ class SAMStreamSession implements SAMMessageSess {
     /** stream id (Long) to StreamSender */
     private final HashMap<Integer,StreamSender> sendersMap = new HashMap<Integer,StreamSender>();
 
-    private final AtomicInteger lastNegativeId = new AtomicInteger();;
+    private final AtomicInteger lastNegativeId = new AtomicInteger();
 
     // Can we create outgoing connections?
     protected final boolean canCreate;
@@ -692,14 +693,15 @@ class SAMStreamSession implements SAMMessageSess {
                 InputStream in = i2pSocket.getInputStream();
 
                 while (stillRunning) {
-                	data.clear();
+                    ((Buffer)data).clear();
                     read = Channels.newChannel(in).read(data);
                     if (read == -1) {
                         if (_log.shouldLog(Log.DEBUG))
                             _log.debug("Handler " + id + ": connection closed");
                         break;
                     }
-                    data.flip();
+                    // not ByteBuffer to avoid Java 8/9 issues with flip()
+                    ((Buffer)data).flip();
                     recv.receiveStreamBytes(id, data);
                 }
             } catch (IOException e) {
