@@ -55,19 +55,19 @@ public class SAMStreamSend {
     private static I2PSSLSocketFactory _sslSocketFactory;
     
     private static final int STREAM=0, DG=1, V1DG=2, RAW=3, V1RAW=4;
-    private static final int MASTER=8;
+    private static final int PRIMARY=8;
     private static final String USAGE = "Usage: SAMStreamSend [-s] [-x] [-m mode] [-v version] [-b samHost] [-p samPort]\n" +
                                         "                     [-o opt=val] [-u user] [-w password] peerDestFile dataDir\n" +
                                         "       modes: stream: 0; datagram: 1; v1datagram: 2; raw: 3; v1raw: 4\n" +
                                         "              default is stream\n" +
                                         "       -s: use SSL\n" +
-                                        "       -x: use master session (forces -v 3.3)\n" +
+                                        "       -x: use primary session (forces -v 3.3)\n" +
                                         "       multiple -o session options are allowed";
 
     public static void main(String args[]) {
         Getopt g = new Getopt("SAM", args, "sxhb:m:o:p:u:v:w:");
         boolean isSSL = false;
-        boolean isMaster = false;
+        boolean isPrimary = false;
         int mode = STREAM;
         String version = "3.3";
         String host = "127.0.0.1";
@@ -83,7 +83,7 @@ public class SAMStreamSend {
                 break;
 
             case 'x':
-                isMaster = true;
+                isPrimary = true;
                 break;
 
             case 'm':
@@ -132,8 +132,8 @@ public class SAMStreamSend {
             System.err.println(USAGE);
             return;
         }
-        if (isMaster) {
-            mode += MASTER;
+        if (isPrimary) {
+            mode += PRIMARY;
             version = "3.3";
         }
         if ((user == null && password != null) ||
@@ -175,8 +175,8 @@ public class SAMStreamSend {
                 _log.debug("Reader created");
             OutputStream out = sock.getOutputStream();
             String ourDest = handshake(out, version, true, eventHandler, mode, user, password, sessionOpts);
-            if (mode >= MASTER)
-                mode -= MASTER;
+            if (mode >= PRIMARY)
+                mode -= PRIMARY;
             if (ourDest == null)
                 throw new IOException("handshake failed");
             if (_log.shouldLog(Log.DEBUG))
@@ -246,10 +246,10 @@ public class SAMStreamSend {
     }
     
     /**
-     * @param isMaster is this the control socket
+     * @param isPrimary is this the control socket
      * @return our b64 dest or null
      */
-    private String handshake(OutputStream samOut, String version, boolean isMaster,
+    private String handshake(OutputStream samOut, String version, boolean isPrimary,
                              SAMEventHandler eventHandler, int mode, String user, String password,
                              String opts) {
         synchronized (samOut) {
@@ -267,7 +267,7 @@ public class SAMStreamSend {
                     _log.debug("Hello reply found: " + hisVersion);
                 if (hisVersion == null) 
                     throw new IOException("Hello failed");
-                if (!isMaster)
+                if (!isPrimary)
                     return "OK";
                 _isV3 = VersionComparator.comp(hisVersion, "3") >= 0;
                 if (_isV3) {
@@ -279,14 +279,14 @@ public class SAMStreamSend {
                         _v3ID = "xx€€xx" + _v3ID;
                     _conOptions = "ID=" + _v3ID;
                 }
-                boolean masterMode;  // are we using v3.3 master session
+                boolean primaryMode;  // are we using v3.3 primary session
                 String command;
-                if (mode >= MASTER) {
-                    masterMode = true;
+                if (mode >= PRIMARY) {
+                    primaryMode = true;
                     command = "ADD";
-                    mode -= MASTER;
+                    mode -= PRIMARY;
                 } else {
-                    masterMode = false;
+                    primaryMode = false;
                     command = "CREATE DESTINATION=TRANSIENT";
                 }
                 String style;
@@ -297,19 +297,19 @@ public class SAMStreamSend {
                 else   // RAW or V1RAW
                     style = "RAW";
 
-                if (masterMode) {
+                if (primaryMode) {
                     if (mode == V1DG || mode == V1RAW)
-                        throw new IllegalArgumentException("v1 dg/raw incompatible with master session");
-                    String req = "SESSION CREATE DESTINATION=TRANSIENT STYLE=MASTER ID=masterSend " + opts + '\n';
+                        throw new IllegalArgumentException("v1 dg/raw incompatible with primary session");
+                    String req = "SESSION CREATE DESTINATION=TRANSIENT STYLE=PRIMARY ID=primarySend " + opts + '\n';
                     samOut.write(req.getBytes("UTF-8"));
                     samOut.flush();
                     if (_log.shouldLog(Log.DEBUG))
-                        _log.debug("SESSION CREATE STYLE=MASTER sent");
+                        _log.debug("SESSION CREATE STYLE=PRIMARY sent");
                     boolean ok = eventHandler.waitForSessionCreateReply();
                     if (!ok) 
-                        throw new IOException("SESSION CREATE STYLE=MASTER failed");
+                        throw new IOException("SESSION CREATE STYLE=PRIMARY failed");
                     if (_log.shouldLog(Log.DEBUG))
-                        _log.debug("SESSION CREATE STYLE=MASTER reply found: " + ok);
+                        _log.debug("SESSION CREATE STYLE=PRIMARY reply found: " + ok);
                     // PORT required even if we aren't listening for this test
                     if (mode != STREAM)
                         opts += " PORT=9999";
@@ -320,7 +320,7 @@ public class SAMStreamSend {
                 if (_log.shouldLog(Log.DEBUG))
                     _log.debug("SESSION " + command + " sent");
                 boolean ok;
-                if (masterMode)
+                if (primaryMode)
                     ok = eventHandler.waitForSessionAddReply();
                 else
                     ok = eventHandler.waitForSessionCreateReply();
@@ -329,7 +329,7 @@ public class SAMStreamSend {
                 if (_log.shouldLog(Log.DEBUG))
                     _log.debug("SESSION " + command + " reply found: " + ok);
 
-                if (masterMode) {
+                if (primaryMode) {
                     // do a bunch more
                     req = "SESSION ADD STYLE=STREAM FROM_PORT=99 ID=stream99\n";
                     samOut.write(req.getBytes("UTF-8"));
@@ -559,7 +559,7 @@ public class SAMStreamSend {
             closed();
             // stop the reader, since we're only doing this once for testing
             // you wouldn't do this in a real application
-            // closing the master socket too fast will kill the data socket flushing through
+            // closing the primary socket too fast will kill the data socket flushing through
             try {
                 Thread.sleep(10000);
             } catch (InterruptedException ie) {}
