@@ -5,6 +5,7 @@ import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.SortedMap;
 import java.util.TreeMap;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
@@ -53,10 +54,11 @@ class Connection {
     private final AtomicInteger _unackedPacketsReceived = new AtomicInteger();
     private long _congestionWindowEnd;
     private volatile long _highestAckedThrough;
+    private volatile int _ssthresh;
     private final boolean _isInbound;
     private boolean _updatedShareOpts;
     /** Packet ID (Long) to PacketLocal for sent but unacked packets */
-    private final Map<Long, PacketLocal> _outboundPackets;
+    private final SortedMap<Long, PacketLocal> _outboundPackets;
     private final PacketQueue _outboundQueue;
     private final ConnectionPacketHandler _handler;
     private ConnectionOptions _options;
@@ -67,8 +69,6 @@ class Connection {
     private final AtomicLong _disconnectScheduledOn = new AtomicLong();
     private long _lastReceivedOn;
     private final ActivityTimer _activityTimer;
-    /** window size when we last saw congestion */
-    private int _lastCongestionSeenAt;
     private long _lastCongestionTime;
     private volatile long _lastCongestionHighestUnacked;
     /** has the other side choked us? */
@@ -84,10 +84,12 @@ class Connection {
     /** how many messages have been resent and not yet ACKed? */
     private final AtomicInteger _activeResends = new AtomicInteger();
     private final ConEvent _connectionEvent;
+    private final RetransmitEvent _retransmitEvent;
     private final int _randomWait;
     private final int _localPort;
     private final int _remotePort;
     private final SimpleTimer2 _timer;
+    private final BandwidthEstimator _bwEstimator;
     
     private final AtomicLong _lifetimeBytesSent = new AtomicLong();
     /** TBD for tcpdump-compatible ack output */
@@ -96,8 +98,8 @@ class Connection {
     private final AtomicLong _lifetimeDupMessageSent = new AtomicLong();
     private final AtomicLong _lifetimeDupMessageReceived = new AtomicLong();
     
-    public static final long MAX_RESEND_DELAY = 45*1000;
-    public static final long MIN_RESEND_DELAY = 100;
+    public static final int MAX_RESEND_DELAY = 45*1000;
+    public static final int MIN_RESEND_DELAY = 100;
 
     /**
      *  Wait up to 5 minutes after disconnection so we can ack/close packets.
@@ -110,7 +112,10 @@ class Connection {
 
     public static final int MAX_WINDOW_SIZE = 128;
     private static final int UNCHOKES_TO_SEND = 8;
-    
+
+    /** Maximum number of packets to retransmit when the timer hits */
+    private static final int MAX_RTX = 16;
+
 /****
     public Connection(I2PAppContext ctx, ConnectionManager manager, SchedulerChooser chooser,
                       PacketQueue queue, ConnectionPacketHandler handler) {
@@ -139,7 +144,8 @@ class Connection {
         _inputStream = new MessageInputStream(_context, _options.getMaxMessageSize(),
                                               _options.getMaxWindowSize(), _options.getInboundBufferSize());
         // FIXME pass through a passive flush delay setting as the 4th arg
-        _outputStream = new MessageOutputStream(_context, timer, _receiver, _options.getMaxMessageSize());
+        _outputStream = new MessageOutputStream(_context, timer, _receiver,
+                                                _options.getMaxMessageSize(), _options.getMaxInitialMessageSize());
         _timer = timer;
         _outboundPackets = new TreeMap<Long, PacketLocal>();
         if (opts != null) {
@@ -156,7 +162,7 @@ class Connection {
         _createdOn = _context.clock().now();
         _congestionWindowEnd = _options.getWindowSize()-1;
         _highestAckedThrough = -1;
-        _lastCongestionSeenAt = MAX_WINDOW_SIZE*2; // lets allow it to grow
+        _ssthresh = ConnectionPacketHandler.MAX_SLOW_START_WINDOW;
         _lastCongestionTime = -1;
         _lastCongestionHighestUnacked = -1;
         _lastReceivedOn = -1;
@@ -165,11 +171,20 @@ class Connection {
         _connectLock = new Object();
         _nextSendLock = new Object();
         _connectionEvent = new ConEvent();
+        _retransmitEvent = new RetransmitEvent();
+        _bwEstimator = new SimpleBandwidthEstimator(ctx, _options);
         _randomWait = _context.random().nextInt(10*1000); // just do this once to reduce usage
         // all createRateStats in ConnectionManager
         if (_log.shouldLog(Log.INFO))
             _log.info("New connection created with options: " + _options);
     }
+
+    /**
+     * @since 0.9.46
+     */
+    int getSSThresh() {
+        return _ssthresh;
+    }
     
     public long getNextOutboundPacketNum() { 
         return _lastSendId.incrementAndGet();
@@ -388,28 +403,27 @@ class Connection {
                 windowSize = _options.getWindowSize();
                 remaining = windowSize - _outboundPackets.size() ;
                 _outboundPackets.notifyAll();
-            }
 
-            if (_isChoking) {
-                packet.setOptionalDelay(Packet.SEND_DELAY_CHOKE);
-                packet.setFlag(Packet.FLAG_DELAY_REQUESTED);
-            } else if (packet.isFlagSet(Packet.FLAG_CLOSE) ||
-                _unchokesToSend.decrementAndGet() > 0 ||
-                // the other end has no idea what our window size is, so
-                // help him out by requesting acks below the 1/3 point,
-                // if remaining < 3, and every 8 minimum.
-                (remaining < 3) ||
-                (remaining < (windowSize + 2) / 3) /* ||
-                (packet.getSequenceNum() % 8 == 0) */ ) {
-                packet.setOptionalDelay(0);
-                packet.setFlag(Packet.FLAG_DELAY_REQUESTED);
-                //if (_log.shouldLog(Log.DEBUG))
-                //    _log.debug("Requesting no ack delay for packet " + packet);
-            } else {
-                // This is somewhat of a waste of time, unless the RTT < 4000,
-                // since the other end limits it to getSendAckDelay()
-                // which is always 2000, but it's good for diagnostics to see what the other end thinks
-                // the RTT is.
+                if (_isChoking) {
+                    packet.setOptionalDelay(Packet.SEND_DELAY_CHOKE);
+                    packet.setFlag(Packet.FLAG_DELAY_REQUESTED);
+                } else if (packet.isFlagSet(Packet.FLAG_CLOSE) ||
+                    _unchokesToSend.decrementAndGet() > 0 ||
+                    // the other end has no idea what our window size is, so
+                    // help him out by requesting acks below the 1/3 point,
+                    // if remaining < 3, and every 8 minimum.
+                    (remaining < 3) ||
+                    (remaining < (windowSize + 2) / 3) /* ||
+                    (packet.getSequenceNum() % 8 == 0) */ ) {
+                    packet.setOptionalDelay(0);
+                    packet.setFlag(Packet.FLAG_DELAY_REQUESTED);
+                    //if (_log.shouldLog(Log.DEBUG))
+                    //    _log.debug("Requesting no ack delay for packet " + packet);
+                } else {
+                    // This is somewhat of a waste of time, unless the RTT < 4000,
+                    // since the other end limits it to getSendAckDelay()
+                    // which is always 2000, but it's good for diagnostics to see what the other end thinks
+                    // the RTT is.
 /**
                 int delay = _options.getRTT() / 2;
                 packet.setOptionalDelay(delay);
@@ -418,16 +432,21 @@ class Connection {
                 if (_log.shouldLog(Log.DEBUG))
                     _log.debug("Requesting ack delay of " + delay + "ms for packet " + packet);
 **/
-            }
+                }
             
-            long timeout = _options.getRTO();
-            if (timeout > MAX_RESEND_DELAY)
-                timeout = MAX_RESEND_DELAY;
-            if (_log.shouldLog(Log.DEBUG))
-                _log.debug("Resend in " + timeout + " for " + packet);
+                int timeout = _options.getRTO();
 
-            // schedules itself
-            new ResendPacketEvent(packet, timeout);
+                // RFC 6298 section 5.1
+                if (_retransmitEvent.scheduleIfNotRunning(timeout)) {
+                    if (_log.shouldLog(Log.DEBUG))
+                        _log.debug(Connection.this + " Resend in " + timeout + " for " + packet);
+                } else {
+                    if (_log.shouldLog(Log.DEBUG))
+                        _log.debug(Connection.this + " timer was already running");
+                }
+
+                packet.setTimeout(timeout);
+            }
         }
 
         // warning, getStatLog() can be null
@@ -492,6 +511,7 @@ class Connection {
         }
         
         List<PacketLocal> acked = null;
+        boolean anyLeft = false;
         synchronized (_outboundPackets) {
             if (!_outboundPackets.isEmpty()) {  // short circuit iterator
               for (Iterator<Map.Entry<Long, PacketLocal>> iter = _outboundPackets.entrySet().iterator(); iter.hasNext(); ) {
@@ -557,10 +577,27 @@ class Connection {
                     _log.info("All outbound packets acked, clearing " + _activeResends);
                 _activeResends.set(0);
             }
+            anyLeft = !_outboundPackets.isEmpty();
             _outboundPackets.notifyAll();
+            if ((acked != null) && (!acked.isEmpty()) ) {
+                _ackSinceCongestion.set(true);
+                _bwEstimator.addSample(acked.size());
+                if (anyLeft) {
+                    // RFC 6298 section 5.3
+                    int rto = _options.getRTO();
+                    _retransmitEvent.pushBackRTO(rto);
+                    
+                    if (_log.shouldLog(Log.DEBUG))
+                        _log.debug(Connection.this + " not all packets acked, pushing timer out " + rto);
+                } else {
+                    // RFC 6298 section 5.2
+                    if (_log.shouldLog(Log.DEBUG))
+                        _log.debug(Connection.this + " all outstanding packets acked, cancelling timer");
+
+                    _retransmitEvent.cancel();
+                }
+            }
         }
-        if ((acked != null) && (!acked.isEmpty()) )
-            _ackSinceCongestion.set(true);
         return acked;
     }
 
@@ -788,6 +825,7 @@ class Connection {
         _outputStream.destroy();
         _receiver.destroy();
         _activityTimer.cancel();
+        _retransmitEvent.cancel();
         _inputStream.streamErrorOccurred(new IOException("Socket closed"));
         
         if (_log.shouldLog(Log.INFO))
@@ -1085,7 +1123,7 @@ class Connection {
             // ...
             // We don't do any of that, but we set the window size to 1, and let the retransmission
             // of packets do the "attempted recovery".
-            getOptions().setWindowSize(1);
+            _options.setWindowSize(1);
         }
     }
     
@@ -1139,13 +1177,10 @@ class Connection {
         return (_lastSendTime > _lastReceivedOn ? _lastSendTime : _lastReceivedOn);
     }
     
-    public int getLastCongestionSeenAt() { return _lastCongestionSeenAt; }
-
     private void congestionOccurred() {
         // if we hit congestion and e.g. 5 packets are resent,
         // dont set the size to (winSize >> 4).  only set the
         if (_ackSinceCongestion.compareAndSet(true,false)) {
-            _lastCongestionSeenAt = _options.getWindowSize();
             _lastCongestionTime = _context.clock().now();
             _lastCongestionHighestUnacked = _lastSendId.get();
         }
@@ -1214,8 +1249,8 @@ class Connection {
     private void resetActivityTimer() {
         long howLong = _options.getInactivityTimeout();
         if (howLong <= 0) {
-            if (_log.shouldLog(Log.DEBUG))
-                _log.debug("Resetting the inactivity timer, but its gone!", new Exception("where did it go?"));
+            //if (_log.shouldLog(Log.DEBUG))
+            //    _log.debug("Resetting the inactivity timer, but its gone!", new Exception("where did it go?"));
             return;
         }
         howLong += _randomWait; // randomize it a bit, so both sides don't do it at once
@@ -1385,14 +1420,169 @@ class Connection {
         buf.append(" sent: ").append(1 + _lastSendId.get());
         buf.append(" rcvd: ").append(1 + _inputStream.getHighestBlockId() - missing);
         buf.append(" ackThru ").append(_highestAckedThrough);
-        
-        buf.append(" maxWin ").append(getOptions().getMaxWindowSize());
-        buf.append(" MTU ").append(getOptions().getMaxMessageSize());
+        buf.append(" ssThresh ").append(_ssthresh); 
+        buf.append(" minRTT ").append(_options.getMinRTT()); 
+        buf.append(" maxWin ").append(_options.getMaxWindowSize());
+        buf.append(" MTU ").append(_options.getMaxMessageSize());
         
         buf.append("]");
         return buf.toString();
     }
 
+    /**
+     *  A single retransmit timer for all packets.
+     *  See RFCs 5681 and 6298.
+     *
+     *  @since 0.9.46
+     */
+    class RetransmitEvent extends SimpleTimer2.TimedEvent {
+
+        private boolean _scheduled;
+
+        RetransmitEvent() {
+            super(_timer);
+        }
+
+        @Override
+        public synchronized boolean cancel() {
+            _scheduled = false;
+            return super.cancel();
+        }
+
+        public synchronized boolean scheduleIfNotRunning(long delay) {
+            if (_scheduled)
+                return false;
+            _scheduled = true;
+            schedule(delay);
+            return true;
+        }
+
+        public synchronized void pushBackRTO(int rto) {
+            if (!_scheduled) {
+                if (_log.shouldWarn())
+                    _log.warn(Connection.this + " timer was not scheduled", new Exception());
+            }
+            reschedule(rto, false);
+        }
+
+        @Override
+        public void timeReached() {
+
+           if (_resetSentOn.get() > 0 || _resetReceived.get() || _finalDisconnect.get()) {
+                if (_log.shouldLog(Log.DEBUG))
+                    _log.debug(Connection.this + " rtx event after close or reset");
+                return;
+            }
+
+            if (_log.shouldLog(Log.DEBUG))
+                _log.debug(Connection.this + " rtx timer timeReached()");
+
+            congestionOccurred();
+
+            // 1. Double RTO and backoff (RFC 6298 section 5.5 & 5.6)
+            final long now = _context.clock().now();
+            pushBackRTO(_options.doubleRTO());
+
+            // 2. cut ssthresh to bandwidth estimate, window to 1
+            List<PacketLocal> toResend = null;
+            synchronized(_outboundPackets) {
+                if (_outboundPackets.isEmpty()) {
+                    if (_log.shouldLog(Log.WARN))
+                        _log.warn(Connection.this + " Retransmission timer hit but nothing transmitted??");
+                    return;
+                }
+
+                PacketLocal oldest = _outboundPackets.get(_outboundPackets.firstKey());
+                if (oldest.getNumSends() == 1) {
+                    if (_log.shouldLog(Log.DEBUG))
+                        _log.debug(Connection.this + " cutting ssthresh and window");
+                    _ssthresh = Math.max( (int)(_bwEstimator.getBandwidthEstimate() * _options.getMinRTT()), 2 );
+                    _ssthresh = Math.min(ConnectionPacketHandler.MAX_SLOW_START_WINDOW, _ssthresh);
+                    _options.setWindowSize(1);
+                } else if (_log.shouldLog(Log.DEBUG))
+                    _log.debug(Connection.this + " not cutting ssthresh and window");
+
+                toResend = new ArrayList<>(_outboundPackets.values());
+                toResend = toResend.subList(0, Math.min(MAX_RTX, (toResend.size() + 1) / 2));
+            }
+
+            // 3. Retransmit up to half of the packets in flight (RFC 6298 section 5.4 and RFC 5681 section 4.3)
+            boolean sentAny = false;
+            for (PacketLocal packet : toResend) {
+                final int nResends = packet.getNumSends();
+                if (packet.getNumSends() > _options.getMaxResends()) {
+                    if (_log.shouldLog(Log.DEBUG))
+                        _log.debug(Connection.this + " packet " + packet + " resent too many times, closing");
+                    packet.cancelled();
+                    disconnect(false);
+                    return;
+                } else if (packet.getNumSends() >= 3 &&
+                           packet.isFlagSet(Packet.FLAG_CLOSE) &&
+                           packet.getPayloadSize() <= 0 &&
+                           getCloseReceivedOn() > 0) {
+                    // Bug workaround to prevent 5 minutes of retransmission
+                    // Routers before 0.9.9 have bugs, they won't ack anything after
+                    // they sent a close. Only send 3 CLOSE packets total, then
+                    // shut down normally.
+                    if (_log.shouldLog(Log.DEBUG))
+                        _log.debug(Connection.this + " too many close resends, closing");
+                    packet.cancelled();
+                    disconnect(false);
+                    return;
+                } else {
+                    
+                    if (_isChoking) {
+                        if (_log.shouldLog(Log.DEBUG))
+                            _log.debug(Connection.this + " packet is choking " + packet);
+                        packet.setOptionalDelay(Packet.SEND_DELAY_CHOKE);
+                        packet.setFlag(Packet.FLAG_DELAY_REQUESTED);
+                    } else if (_unchokesToSend.decrementAndGet() > 0) {
+                        if (_log.shouldLog(Log.DEBUG))
+                            _log.debug(Connection.this + " packet is unchoking " + packet);
+                        // don't worry about wrapping around
+                        packet.setOptionalDelay(0);
+                        packet.setFlag(Packet.FLAG_DELAY_REQUESTED);
+                    } else {
+                        if (_log.shouldLog(Log.DEBUG))
+                            _log.debug(Connection.this + " packet clearing flag " + packet);
+                        // clear flag
+                        packet.setFlag(Packet.FLAG_DELAY_REQUESTED, false);
+                    }
+
+                    // this seems unnecessary to send the MSS again:
+                    //_packet.setOptionalMaxSize(_options.getMaxMessageSize());
+                    // bugfix release 0.7.8, we weren't dividing by 1000
+                    packet.setResendDelay(_options.getResendDelay() / 1000);
+                    if (packet.getReceiveStreamId() <= 0)
+                        packet.setReceiveStreamId(_receiveStreamId.get());
+                    if (packet.getSendStreamId() <= 0)
+                        packet.setSendStreamId(_sendStreamId.get());
+
+                    packet.setTimeout(_options.getRTO());
+                    
+
+                    if (_outboundQueue.enqueue(packet)) {
+                        if (_log.shouldLog(Log.INFO)) 
+                            _log.info(Connection.this + " resent packet " + packet);
+                        if (nResends == 1)
+                            _activeResends.incrementAndGet();
+                        sentAny = true;
+                    } else if (_log.shouldLog(Log.DEBUG)) 
+                       _log.debug(Connection.this + " could not resend packet " + packet);
+                }
+            }
+
+            if (sentAny) {
+                _lastSendTime = now;
+                resetActivityTimer();
+                windowAdjusted();
+            }
+
+        }
+    }
+
+
+
     /**
      * fired to reschedule event notification
      */
@@ -1411,30 +1601,41 @@ class Connection {
     
     /**
      * If we have been explicitly NACKed three times, retransmit the packet even if
-     * there are other packets in flight. 3 takes forever, let's try 2.
+     * there are other packets in flight.
      *
      */
     static final int FAST_RETRANSMIT_THRESHOLD = 3;
     
     /**
-     * Coordinate the resends of a given packet
+     * A new ResendPacketEvent.
+     * @since 0.9.46
+     */
+    ResendPacketEvent newResendPacketEvent(PacketLocal packet) {
+        return new ResendPacketEvent(packet);
+    }
+    
+    /**
+     * This is not normally scheduled. It's now used only for fastRetransmit(),
+     * where it's scheduled with a delay of zero to put it on the timer queue.
+     * Timeout retransmissions are handled by RetransmitEvent above.
      */
     class ResendPacketEvent extends SimpleTimer2.TimedEvent {
         private final PacketLocal _packet;
-        private long _nextSend;
 
-        public ResendPacketEvent(PacketLocal packet, long delay) {
+        public ResendPacketEvent(PacketLocal packet) {
             super(_timer);
             _packet = packet;
-            _nextSend = delay + _context.clock().now();
-            packet.setResendPacketEvent(ResendPacketEvent.this);
-            schedule(delay);
         }
         
-        public long getNextSendTime() { return _nextSend; }
-
         public void timeReached() { retransmit(); }
 
+        /**
+         * @since 0.9.46
+         */
+        void fastRetransmit() {
+            reschedule(0);
+        }
+
         /**
          * Retransmit the packet if we need to.  
          *
@@ -1454,43 +1655,8 @@ class Connection {
                 _packet.cancelled();
                 return false;
             }
-            
-            //if (_log.shouldLog(Log.DEBUG))
-            //    _log.debug("Resend period reached for " + _packet);
-            boolean resend = false;
-            boolean isLowest = false;
-            synchronized (_outboundPackets) {
-                // allow appx. half the window to be "lowest" and be active resends, minimum of 3
-                // Note: we should really pick the N lowest, not the lowest one + N more who
-                // happen to get here next, as the timers get out-of-order esp. after fast retx
-                if (_packet.getSequenceNum() == _highestAckedThrough + 1 ||
-                    _packet.getNumSends() > 1 ||
-                    _activeResends.get() < Math.max(3, (_options.getWindowSize() + 1) / 2))
-                    isLowest = true;
-                if (_outboundPackets.containsKey(Long.valueOf(_packet.getSequenceNum())))
-                    resend = true;
-            }
-            if ( (resend) && (_packet.getAckTime() <= 0) ) {
-                boolean fastRetransmit = ( (_packet.getNACKs() >= FAST_RETRANSMIT_THRESHOLD) && (_packet.getNumSends() == 1));
-                if ( (!isLowest) && (!fastRetransmit) ) {
-                    // we want to resend this packet, but there are already active
-                    // resends in the air and we dont want to make a bad situation 
-                    // worse.  wait another second
-                    // BUG? seq# = 0, activeResends = 0, loop forever - why?
-                    // also seen with seq# > 0. Is the _activeResends count reliable?
-                    if (_log.shouldLog(Log.INFO))
-                        _log.info("Delaying resend of " + _packet + " with " 
-                                  + _activeResends + " active resend, "
-                                  + _outboundPackets.size() + " unacked, window size = " + _options.getWindowSize());
-                    forceReschedule(1333);
-                    _nextSend = 1333 + _context.clock().now();
-                    return false;
-                }
-                
-                // It's the lowest, or it's fast retransmit time. Resend the packet.
 
-                if (fastRetransmit)
-                    _context.statManager().addRateData("stream.fastRetransmit", _packet.getLifetime(), _packet.getLifetime());
+               _context.statManager().addRateData("stream.fastRetransmit", _packet.getLifetime(), _packet.getLifetime());
                 
                 // revamp various fields, in case we need to ack more, etc
                 // updateAcks done in enqueue()
@@ -1508,39 +1674,41 @@ class Connection {
                 }
 
                 // this seems unnecessary to send the MSS again:
-                //_packet.setOptionalMaxSize(getOptions().getMaxMessageSize());
+                //_packet.setOptionalMaxSize(_options.getMaxMessageSize());
                 // bugfix release 0.7.8, we weren't dividing by 1000
-                _packet.setResendDelay(getOptions().getResendDelay() / 1000);
+                _packet.setResendDelay(_options.getResendDelay() / 1000);
                 if (_packet.getReceiveStreamId() <= 0)
                     _packet.setReceiveStreamId(_receiveStreamId.get());
                 if (_packet.getSendStreamId() <= 0)
                     _packet.setSendStreamId(_sendStreamId.get());
                 
-                int newWindowSize = getOptions().getWindowSize();
+                int newWindowSize = _options.getWindowSize();
 
                 if (_isChoked) {
                     congestionOccurred();
-                    getOptions().setWindowSize(1);
+                    _options.setWindowSize(1);
                 } else if (_ackSinceCongestion.get()) {
                     // only shrink the window once per window
                     if (_packet.getSequenceNum() > _lastCongestionHighestUnacked) {
                         congestionOccurred();
                         _context.statManager().addRateData("stream.con.windowSizeAtCongestion", newWindowSize, _packet.getLifetime());
-                        newWindowSize /= 2;
-                        if (newWindowSize <= 0)
-                            newWindowSize = 1;
-                        
                         // The timeout for _this_ packet will be doubled below, but we also
                         // need to double the RTO for the _next_ packets.
                         // See RFC 6298 section 5 item 5.5
                         // This prevents being stuck at a window size of 1, retransmitting every packet,
                         // never updating the RTT or RTO.
-                        getOptions().doubleRTO();
-                        getOptions().setWindowSize(newWindowSize);
+                        _options.doubleRTO();
+
+                        if (_packet.getNumSends() == 1) {
+                            _ssthresh = Math.max( (int)(_bwEstimator.getBandwidthEstimate() * _options.getMinRTT()), 2 );
+                            _ssthresh = Math.min(ConnectionPacketHandler.MAX_SLOW_START_WINDOW, _ssthresh);
+                            int wsize = _options.getWindowSize();
+                            _options.setWindowSize(Math.min(_ssthresh, wsize));
+                        }
 
                         if (_log.shouldLog(Log.INFO))
                             _log.info("Congestion, resending packet " + _packet.getSequenceNum() + " (new windowSize " + newWindowSize 
-                                      + "/" + getOptions().getWindowSize() + ") for " + Connection.this.toString());
+                                      + "/" + _options.getWindowSize() + ") for " + Connection.this.toString());
 
                         windowAdjusted();
                     }
@@ -1578,22 +1746,24 @@ class Connection {
                     disconnect(true);
                 } else {
                     //long timeout = _options.getResendDelay() << numSends;
-                    long rto = _options.getRTO();
-                    if (rto < MIN_RESEND_DELAY)
-                        rto = MIN_RESEND_DELAY;
-                    long timeout = rto << (numSends-1);
+                    int timeout = _options.getRTO();
                     if ( (timeout > MAX_RESEND_DELAY) || (timeout <= 0) )
                         timeout = MAX_RESEND_DELAY;
                     // set this before enqueue() as it passes it on to the router
-                    _nextSend = timeout + _context.clock().now();
+                    _packet.setTimeout(timeout);
 
                     if (_outboundQueue.enqueue(_packet)) {
+                        if (_retransmitEvent.scheduleIfNotRunning(timeout)) {
+                            if (_log.shouldLog(Log.DEBUG))
+                                _log.debug(Connection.this + " fast retransmit and schedule timer");
+                        }
+
                         // first resend for this packet ?
                         if (numSends == 2)
                             _activeResends.incrementAndGet();
                         if (_log.shouldLog(Log.INFO))
                             _log.info("Resent packet " +
-                                  (fastRetransmit ? "(fast) " : "(timeout) ") +
+                                  "(fast) " +
                                   _packet +
                                   " next resend in " + timeout + "ms" +
                                   " activeResends: " + _activeResends + 
@@ -1606,7 +1776,6 @@ class Connection {
                         resetActivityTimer();
                     }
 
-                    forceReschedule(timeout);
                 }
                 
                 // acked during resending (... or somethin') ????????????
@@ -1618,12 +1787,6 @@ class Connection {
                 }
 
                 return true;
-            } else {
-                //if (_log.shouldLog(Log.DEBUG))
-                //    _log.debug("Packet acked before resend (resend="+ resend + "): " 
-                //               + _packet + " on " + Connection.this);
-                return false;
-            }
         }
     }
 }
