@@ -62,16 +62,24 @@ class MessageOutputStream extends OutputStream {
 
     /** */
     public MessageOutputStream(I2PAppContext ctx, SimpleTimer2 timer,
-                               DataReceiver receiver, int bufSize) {
-        this(ctx, timer, receiver, bufSize, DEFAULT_PASSIVE_FLUSH_DELAY);
+                               DataReceiver receiver, int bufSize, int initBufSize) {
+        this(ctx, timer, receiver, bufSize, initBufSize, DEFAULT_PASSIVE_FLUSH_DELAY);
     }
 
     public MessageOutputStream(I2PAppContext ctx, SimpleTimer2 timer,
-                               DataReceiver receiver, int bufSize, int passiveFlushDelay) {
+                               DataReceiver receiver, int bufSize, int initBufSize, int passiveFlushDelay) {
         super();
+        // we only use two buffer sizes to prevent an attack
+        // where we end up with a thousand caches
+        if (bufSize < ConnectionOptions.DEFAULT_MAX_MESSAGE_SIZE) {
+            bufSize = ConnectionOptions.DEFAULT_MAX_MESSAGE_SIZE;
+        } else if (bufSize > ConnectionOptions.DEFAULT_MAX_MESSAGE_SIZE &&
+                   bufSize < ConnectionOptions.DEFAULT_MAX_MESSAGE_SIZE_RATCHET) {
+            bufSize = ConnectionOptions.DEFAULT_MAX_MESSAGE_SIZE_RATCHET;
+        }
         _dataCache = ByteCache.getInstance(128, bufSize);
         _originalBufferSize = bufSize;
-        _currentBufferSize = bufSize;
+        _currentBufferSize = initBufSize;
         _context = ctx;
         _log = ctx.logManager().getLog(MessageOutputStream.class);
         _buf = _dataCache.acquire().getData(); // new byte[bufSize];
@@ -79,7 +87,6 @@ class MessageOutputStream extends OutputStream {
         _dataLock = new Object();
         _writeTimeout = -1;
         _passiveFlushDelay = passiveFlushDelay;
-        _nextBufferSize = 0;
         //_sendPeriodBeginTime = ctx.clock().now();
         //_context.statManager().createRateStat("stream.sendBps", "How fast we pump data through the stream", "Stream", new long[] { 60*1000, 5*60*1000, 60*60*1000 });
         _flusher = new Flusher(timer);
@@ -119,7 +126,7 @@ class MessageOutputStream extends OutputStream {
             _log.debug("write(b[], " + off + ", " + len + ") ");
         int cur = off;
         int remaining = len;
-        long begin = _context.clock().now();
+        long begin = _log.shouldDebug() ? _context.clock().now() : 0;
         while (remaining > 0) {
             WriteStatus ws = null;
             if (_closed.get()) throw new IOException("Output stream closed");
@@ -189,9 +196,11 @@ class MessageOutputStream extends OutputStream {
                     _log.debug("Queued " + len + " without sending to the receiver");
             }
         }
-        long elapsed = _context.clock().now() - begin;
-        if ( (elapsed > 10*1000) && (_log.shouldLog(Log.INFO)) )
-            _log.info("took " + elapsed + "ms to write to the stream?", new Exception("foo"));
+        if (_log.shouldDebug()) {
+            long elapsed = _context.clock().now() - begin;
+            if (elapsed > 10*1000)
+                _log.info("took " + elapsed + "ms to write to the stream?", new Exception("foo"));
+        }
         throwAnyError();
         //updateBps(len);
     }
@@ -244,9 +253,11 @@ class MessageOutputStream extends OutputStream {
      */
     private class Flusher extends SimpleTimer2.TimedEvent {
         private boolean _enqueued;
+
         public Flusher(SimpleTimer2 timer) { 
             super(timer);
         }
+
         public void enqueue() {
             // no need to be overly worried about duplicates - it would just 
             // push it further out
@@ -258,12 +269,13 @@ class MessageOutputStream extends OutputStream {
                 forceReschedule(_passiveFlushDelay);
                 if (_log.shouldLog(Log.DEBUG))
                     _log.debug("Enqueueing the flusher for " + _passiveFlushDelay + "ms out");
+                _enqueued = true;
             } else {
                 if (_log.shouldLog(Log.DEBUG))
                     _log.debug("NOT enqueing the flusher");
             }
-            _enqueued = true;
         }
+
         public void timeReached() {
             if (_closed.get())
                 return;
@@ -334,7 +346,7 @@ class MessageOutputStream extends OutputStream {
      *  @@since 0.8.1
      */
     private void flush(boolean wait_for_accept_only) throws IOException {
-        long begin = _context.clock().now();
+        long begin = _log.shouldDebug() ? _context.clock().now() : 0;
         WriteStatus ws = null;
         if (_log.shouldLog(Log.INFO) && _valid > 0)
             _log.info("flush() valid = " + _valid);
@@ -388,9 +400,11 @@ class MessageOutputStream extends OutputStream {
         else if (ws.writeFailed())
             throw new IOException("Write failed");
         
-        long elapsed = _context.clock().now() - begin;
-        if ( (elapsed > 10*1000) && (_log.shouldLog(Log.DEBUG)) )
-            _log.debug("took " + elapsed + "ms to flush the stream?\n" + ws, new Exception("bar"));
+        if (_log.shouldDebug()) {
+            long elapsed = _context.clock().now() - begin;
+            if (elapsed > 10*1000)
+                _log.debug("took " + elapsed + "ms to flush the stream?\n" + ws, new Exception("bar"));
+        }
         throwAnyError();
     }
     
