@@ -12,6 +12,7 @@ import net.i2p.I2PAppContext;
 import net.i2p.client.I2PClient;
 import net.i2p.client.I2PClientFactory;
 import net.i2p.client.I2PSession;
+import net.i2p.client.streaming.IncomingConnectionFilter;
 
 public class PingIT extends TestCase {
 
@@ -19,7 +20,8 @@ public class PingIT extends TestCase {
     public void test() throws Exception {
         I2PAppContext context = I2PAppContext.getGlobalContext();
         I2PSession session = createSession();
-        ConnectionManager mgr = new ConnectionManager(context, session, new ConnectionOptions());
+        ConnectionManager mgr = new ConnectionManager(
+            context, session, new ConnectionOptions(), IncomingConnectionFilter.ALLOW);
         for (int i = 0; i < 10; i++) {
             boolean ponged = mgr.ping(session.getMyDestination(), 2*1000);
             assertTrue(ponged);
