@@ -19,26 +19,24 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *  
- * $Revision: 1.1 $
+ * $Revision: 1.2 $
  */
 
 package i2p.susi.dns;
 
+import java.io.Serializable;
 import java.util.Comparator;
 
-public 	class AddressByNameSorter implements Comparator
+public class AddressByNameSorter implements Comparator<AddressBean>, Serializable
 {
-	public int compare(Object arg0, Object arg1)
+	public int compare(AddressBean a, AddressBean b)
 	{
-		AddressBean a = (AddressBean)arg0;
-		AddressBean b = (AddressBean)arg1;
-		
 		if( a == null )
 			return 1;
 		
 		if( b == null )
 			return -1;
 		
-		return a.getName().compareToIgnoreCase(b.getName());
+		return a.getDisplayName().compareToIgnoreCase(b.getDisplayName());
 	}	
 }
