@@ -23,15 +23,15 @@
  */
 package i2p.susi.webmail.encoding;
 
-import i2p.susi.util.HexTable;
 import i2p.susi.util.Buffer;
+import i2p.susi.util.HexTable;
 import i2p.susi.util.MemoryBuffer;
+import i2p.susi.util.StringBuilderWriter;
 
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.io.StringWriter;
 import java.io.Writer;
 
 /**
@@ -48,7 +48,7 @@ public class QuotedPrintable extends Encoding {
 
 	public String encode( byte in[] ) throws EncodingException {
 		try {
-			StringWriter strBuf = new StringWriter();
+			Writer strBuf = new StringBuilderWriter();
 			encode(new ByteArrayInputStream(in), strBuf);
 			return strBuf.toString();
 		} catch (IOException e) {
