@@ -46,6 +46,7 @@ import java.util.zip.CRC32;
 import java.util.zip.Deflater;
 
 import net.i2p.I2PAppContext;
+import net.i2p.util.ByteArrayStream;
 import net.i2p.util.ByteCache;
 import net.i2p.util.FileUtil;
 import net.i2p.util.OrderedProperties;
@@ -283,9 +284,8 @@ public class DataHelper {
             }
             if (baos.size() > 65535)
                 throw new DataFormatException("Properties too big (65535 max): " + baos.size());
-            byte propBytes[] = baos.toByteArray();
-            writeLong(rawStream, 2, propBytes.length);
-            rawStream.write(propBytes);
+            writeLong(rawStream, 2, baos.size());
+            baos.writeTo(rawStream);
         } else {
             writeLong(rawStream, 2, 0);
         }
@@ -302,20 +302,23 @@ public class DataHelper {
      *
      * Properties from the defaults table of props (if any) are not written out by this method.
      *
-     * @deprecated unused
-     *
      * @param target returned array as specified in data structure spec
      * @param props source may be null
      * @return new offset
      * @throws DataFormatException if any string is over 255 bytes long, or if the total length
      *                             (not including the two length bytes) is greater than 65535 bytes.
+     * @since un-deprecated in 0.9.48
      */
-    @Deprecated
     public static int toProperties(byte target[], int offset, Properties props) throws DataFormatException, IOException {
-        if (props != null) {
-            OrderedProperties p = new OrderedProperties();
-            p.putAll(props);
-            ByteArrayOutputStream baos = new ByteArrayOutputStream(p.size() * 64);
+        if (props != null && !props.isEmpty()) {
+            Properties p;
+            if (props instanceof OrderedProperties) {
+                p = props;
+            } else {
+                p = new OrderedProperties();
+                p.putAll(props);
+            }
+            ByteArrayStream baos = new ByteArrayStream(p.size() * 64);
             for (Map.Entry<Object, Object> entry : p.entrySet()) {
                 String key = (String) entry.getKey();
                 String val = (String) entry.getValue();
@@ -326,11 +329,10 @@ public class DataHelper {
             }
             if (baos.size() > 65535)
                 throw new DataFormatException("Properties too big (65535 max): " + baos.size());
-            byte propBytes[] = baos.toByteArray();
-            toLong(target, offset, 2, propBytes.length);
+            toLong(target, offset, 2, baos.size());
             offset += 2;
-            System.arraycopy(propBytes, 0, target, offset, propBytes.length);
-            offset += propBytes.length;
+            baos.copyTo(target, offset);
+            offset += baos.size();
             return offset;
         } else {
             toLong(target, offset, 2, 0);
@@ -1902,7 +1904,8 @@ public class DataHelper {
      */
     public static byte[] decompress(byte orig[], int offset, int length) throws IOException {
         if (orig == null) return orig;
-        if (length < 23)
+        // normal overhead is 23 bytes, but a compress of zero bytes is 20 bytes
+        if (length < 20)
             throw new IOException("length");
         if (length < 65559 && orig[offset + 10] == 0x01)
             return zeroDecompress(orig, offset, length);
