@@ -9,7 +9,6 @@ package net.i2p.data.i2cp;
  *
  */
 
-import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 
@@ -17,6 +16,7 @@ import net.i2p.data.DataFormatException;
 import net.i2p.data.LeaseSet;
 import net.i2p.data.PrivateKey;
 import net.i2p.data.SigningPrivateKey;
+import net.i2p.util.ByteArrayStream;
 
 /**
  * Defines the message a client sends to a router when authorizing
@@ -105,7 +105,7 @@ public class CreateLeaseSetMessage extends I2CPMessageImpl {
                  + _signingPrivateKey.length()
                  + PrivateKey.KEYSIZE_BYTES
                  + _leaseSet.size();
-        ByteArrayOutputStream os = new ByteArrayOutputStream(size);
+        ByteArrayStream os = new ByteArrayStream(size);
         try {
             _sessionId.writeBytes(os);
             _signingPrivateKey.writeBytes(os);
