@@ -38,6 +38,11 @@ public class RateStat {
             _rates[i] = rate;
         }
     }
+
+    /**
+     *  Sets the default stat log for this RateStat.
+     *  Deprecated, unused, to be disabled in a future release.
+     */
     public void setStatLog(StatLog sl) { _statLog = sl; }
     
     /** 
@@ -177,22 +182,37 @@ public class RateStat {
                 && DataHelper.eq(getName(), rs.getName());
     }
 
+    /**
+     * Includes comment lines
+     */
     public void store(OutputStream out, String prefix) throws IOException {
+        store(out, prefix, true);
+    }
+
+    /**
+     * @param addComments add comment lines to the output
+     * @since 0.9.41
+     */
+    public void store(OutputStream out, String prefix, boolean addComments) throws IOException {
         StringBuilder buf = new StringBuilder(1024);
-        buf.append(NL);
-        buf.append("################################################################################").append(NL);
-        buf.append("# Rate: ").append(_groupName).append(": ").append(_statName).append(NL);
-        buf.append("# ").append(_description).append(NL);
-        buf.append("# ").append(NL).append(NL);
-        out.write(buf.toString().getBytes("UTF-8"));
-        buf.setLength(0);
-        for (Rate r: _rates){
-            buf.append("#######").append(NL);
-            buf.append("# Period : ").append(DataHelper.formatDuration(r.getPeriod())).append(" for rate ")
-                .append(_groupName).append(" - ").append(_statName).append(NL);
+        if (addComments) {
             buf.append(NL);
+            buf.append("################################################################################").append(NL);
+            buf.append("# Rate: ").append(_groupName).append(": ").append(_statName).append(NL);
+            buf.append("# ").append(_description).append(NL);
+            buf.append("# ").append(NL).append(NL);
+            out.write(buf.toString().getBytes("UTF-8"));
+            buf.setLength(0);
+        }
+        for (Rate r: _rates){
+            if (addComments) {
+                buf.append("#######").append(NL);
+                buf.append("# Period : ").append(DataHelper.formatDuration(r.getPeriod())).append(" for rate ")
+                    .append(_groupName).append(" - ").append(_statName).append(NL);
+                buf.append(NL);
+            }
             String curPrefix = prefix + "." + DataHelper.formatDuration(r.getPeriod());
-            r.store(curPrefix, buf);
+            r.store(curPrefix, buf, addComments);
             out.write(buf.toString().getBytes("UTF-8"));
             buf.setLength(0);
         }
