@@ -14,7 +14,7 @@ package net.i2p.util;
  *
  */
 class LogLimit {
-    private String _rootName;
+    private final String _rootName;
     private int _limit;
 
     public LogLimit(String name, int limit) {
@@ -37,6 +37,23 @@ class LogLimit {
     public boolean matches(Log log) {
         String name = log.getName();
         if (name == null) return false;
-        return name.startsWith(_rootName);
+        //return name.startsWith(_rootName);
+        // exact match or higher in class hierarchy
+        // no longer allow foo.bar to match foo.barf
+        return name.startsWith(_rootName) &&
+               (name.length() == _rootName.length() ||
+                name.charAt(_rootName.length()) == '.');
     }
-}
\ No newline at end of file
+
+    @Override
+    public int hashCode() {
+        return _rootName.hashCode();
+    }
+
+    @Override
+    public boolean equals(Object o) {
+        if (o == null || !(o instanceof LogLimit))
+            return false;
+        return _rootName.equals(((LogLimit) o).getRootName());
+    }
+}
