@@ -95,8 +95,9 @@ public class TunnelGatewayMessage extends FastI2NPMessageImpl {
         curIndex += 2;
         // where is this coming from?
         if (curIndex + _msgData.length > out.length) {
-            _log.log(Log.ERROR, "output buffer too small idx: " + curIndex + " len: " + _msgData.length + " outlen: " + out.length);
-            throw new I2NPMessageException("Too much data to write out (id=" + _tunnelId + " data=" + _msg + ")");
+            String txt = "output buffer too small idx: " + curIndex + " len: " + _msgData.length + " outlen: " + out.length;
+            _log.error(txt);
+            throw new I2NPMessageException(txt);
         }
         System.arraycopy(_msgData, 0, out, curIndex, _msgData.length);
         curIndex += _msgData.length;
