@@ -25,16 +25,20 @@ import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
 
+import net.i2p.app.ClientAppManager;
 import net.i2p.data.Base64;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
 import net.i2p.data.router.RouterAddress;
 import net.i2p.data.router.RouterInfo;
 import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
+import net.i2p.update.UpdateManager;
+import net.i2p.update.UpdateType;
 import net.i2p.util.Addresses;
 import net.i2p.util.ConcurrentHashSet;
 import net.i2p.util.LHMCache;
 import net.i2p.util.Log;
+import net.i2p.util.SimpleTimer2;
 import net.i2p.util.Translate;
 
 /**
@@ -73,19 +77,23 @@ public class Blocklist {
     private final RouterContext _context;
     private long _blocklist[];
     private int _blocklistSize;
+    private long _countryBlocklist[];
+    private int _countryBlocklistSize;
     private final Object _lock = new Object();
     private Entry _wrapSave;
     private final Set<Hash> _inProcess = new HashSet<Hash>(4);
     private final File _blocklistFeedFile;
     private boolean _started;
     // temp
-    private Map<Hash, String> _peerBlocklist = new HashMap<Hash, String>(4);
+    private final Map<Hash, String> _peerBlocklist = new HashMap<Hash, String>(4);
     
     private static final String PROP_BLOCKLIST_ENABLED = "router.blocklist.enable";
     private static final String PROP_BLOCKLIST_DETAIL = "router.blocklist.detail";
     private static final String PROP_BLOCKLIST_FILE = "router.blocklist.file";
-    private static final String BLOCKLIST_FILE_DEFAULT = "blocklist.txt";
+    public static final String BLOCKLIST_FILE_DEFAULT = "blocklist.txt";
     private static final String BLOCKLIST_FEED_FILE = "docs/feed/blocklist/blocklist.txt";
+    /** @since 0.9.48 */
+    public static final String BLOCKLIST_COUNTRY_FILE = "blocklist-country.txt";
 
     /**
      *  Limits of transient (in-memory) blocklists.
@@ -100,6 +108,17 @@ public class Blocklist {
 
     private static final Object DUMMY = Integer.valueOf(0);    
 
+    /**
+     *  For Update Manager
+     *  @since 0.9.48
+     */
+    public static final String ID_FEED = "feed";
+    private static final String ID_SYSTEM = "system";
+    private static final String ID_LOCAL = "local";
+    private static final String ID_COUNTRY = "country";
+    private static final String ID_USER = "user";
+
+
     /**
      *  Router MUST call startup()
      */
@@ -121,6 +140,7 @@ public class Blocklist {
      *  $I2P/blocklist.txt
      *  ~/.i2p/blocklist.txt
      *  ~/.i2p/docs/feed/blocklist/blocklist.txt
+     *  ~/.i2p/blocklist-countries.txt
      *  File if specified with router.blocklist.file
      */
     public synchronized void startup() {
@@ -129,24 +149,26 @@ public class Blocklist {
         _started = true;
         if (! _context.getBooleanPropertyDefaultTrue(PROP_BLOCKLIST_ENABLED))
             return;
-        List<File> files = new ArrayList<File>(4);
+        List<BLFile> files = new ArrayList<BLFile>(5);
 
         // install dir
         File blFile = new File(_context.getBaseDir(), BLOCKLIST_FILE_DEFAULT);
-        files.add(blFile);
+        files.add(new BLFile(blFile, ID_SYSTEM));
         // config dir
         if (!_context.getConfigDir().equals(_context.getBaseDir())) {
             blFile = new File(_context.getConfigDir(), BLOCKLIST_FILE_DEFAULT);
-            files.add(blFile);
+            files.add(new BLFile(blFile, ID_LOCAL));
         }
-        files.add(_blocklistFeedFile);
+        files.add(new BLFile(_blocklistFeedFile, ID_FEED));
+        blFile = new File(_context.getConfigDir(), BLOCKLIST_COUNTRY_FILE);
+        files.add(new BLFile(blFile, ID_COUNTRY));
         // user specified
         String file = _context.getProperty(PROP_BLOCKLIST_FILE);
         if (file != null && !file.equals(BLOCKLIST_FILE_DEFAULT)) {
             blFile = new File(file);
             if (!blFile.isAbsolute())
                  blFile = new File(_context.getConfigDir(), file);
-            files.add(blFile);
+            files.add(new BLFile(blFile, ID_USER));
         }
         Job job = new ReadinJob(files);
         // Run immediately, so it's initialized before netdb.
@@ -158,13 +180,51 @@ public class Blocklist {
         _context.jobQueue().addJob(job);
     }
 
+    /**
+     *  @since 0.9.48
+     */
+    private static class BLFile {
+        public final File file;
+        public final String id;
+        public long version;
+        public BLFile(File f, String s) { file = f; id = s; }
+    }
+
+    /**
+     *  Delay telling update manager until it's there
+     *  @since 0.9.48
+     */
+    private class VersionNotifier extends SimpleTimer2.TimedEvent {
+        public final List<BLFile> blfs;
+
+        public VersionNotifier(List<BLFile> bf) {
+            super(_context.simpleTimer2(), 2*60*1000L);
+            blfs = bf;
+        }
+
+        public void timeReached() {
+            ClientAppManager cmgr = _context.clientAppManager();
+            if (cmgr != null) {
+                UpdateManager umgr = (UpdateManager) cmgr.getRegisteredApp(UpdateManager.APP_NAME);
+                if (umgr != null) {
+                    for (BLFile blf : blfs) {
+                        if (blf.version > 0)
+                               umgr.notifyInstalled(UpdateType.BLOCKLIST, blf.id, Long.toString(blf.version));
+                    }
+                } else {
+                    _log.warn("No update manager");
+                }
+            }
+        }
+    }
+
     private class ReadinJob extends JobImpl {
-        private final List<File> _files;
+        private final List<BLFile> _files;
 
         /**
          *  @param files not necessarily existing, but avoid dups
          */
-        public ReadinJob (List<File> files) {
+        public ReadinJob (List<BLFile> files) {
             super(_context);
             _files = files;
         }
@@ -172,48 +232,37 @@ public class Blocklist {
         public String getName() { return "Read Blocklist"; }
 
         public void runJob() {
-            allocate(_files);
-            if (_blocklist == null)
-                return;
-            int ccount = process();
-            if (_blocklist == null)
-                return;
-            if (ccount <= 0) {
-                disable();
-                return;
-            }
-            merge(ccount);
-         /**** debug, and now run before netdb is initialized anyway
-            if (_log.shouldLog(Log.WARN)) {
-                if (_blocklistSize <= 0)
+            synchronized (_lock) {
+                _blocklist = allocate(_files);
+                if (_blocklist == null)
+                    return;
+                int ccount = process();
+                if (_blocklist == null)
+                    return;
+                if (ccount <= 0) {
+                    disable();
                     return;
-                FloodfillNetworkDatabaseFacade fndf = (FloodfillNetworkDatabaseFacade) _context.netDb();
-                int count = 0;
-                for (RouterInfo ri : fndf.getKnownRouterData()) {
-                    Hash peer = ri.getIdentity().getHash();
-                    if (isBlocklisted(peer))
-                        count++;
                 }
-                if (count > 0)
-                    _log.warn("Blocklisted " + count + " routers in the netDb");
+                _blocklistSize = merge(_blocklist, ccount);
+                // we're done with _peerBlocklist, but leave it
+                // in case we need it for a later readin
+                //_peerBlocklist = null;
             }
-          ****/
-            _peerBlocklist = null;
+            // schedules itself
+            new VersionNotifier(_files);
         }
 
         private int process() {
             int count = 0;
-            synchronized (_lock) {
                 try {
-                    for (File f : _files) {
-                        count = readBlocklistFile(f, count);
+                    for (BLFile blf : _files) {
+                        count = readBlocklistFile(blf, _blocklist, count);
                     }
                 } catch (OutOfMemoryError oom) {
                     _log.log(Log.CRIT, "OOM processing the blocklist");
                     disable();
                     return 0;
                 }
-            }
             for (Hash peer : _peerBlocklist.keySet()) {
                 String reason;
                 String comment = _peerBlocklist.get(peer);
@@ -228,6 +277,33 @@ public class Blocklist {
         }
     }
 
+    /**
+     *  The blocklist-country.txt file was created or updated.
+     *  Read it in. Not required normally, as the country file
+     *  is read by startup().
+     *  @since 0.9.48
+     */
+    public synchronized void addCountryFile() {
+        File blFile = new File(_context.getConfigDir(), BLOCKLIST_COUNTRY_FILE);
+        BLFile blf = new BLFile(blFile, ID_COUNTRY);
+        List<BLFile> c = Collections.singletonList(blf);
+        long[] cb = allocate(c);
+        if (cb == null)
+            return;
+        int count = readBlocklistFile(blf, cb, 0);
+        if (count <= 0)
+            return;
+        ClientAppManager cmgr = _context.clientAppManager();
+        if (cmgr != null) {
+            UpdateManager umgr = (UpdateManager) cmgr.getRegisteredApp(UpdateManager.APP_NAME);
+            if (umgr != null)
+                umgr.notifyInstalled(UpdateType.BLOCKLIST, ID_COUNTRY, Long.toString(blFile.lastModified()));
+        }
+        count = merge(cb, count);
+        _countryBlocklistSize = count;
+        _countryBlocklist = cb;
+    }
+
     public void disable() {
         // hmm better block out any checks in process
         synchronized (_lock) {
@@ -237,19 +313,19 @@ public class Blocklist {
     }
 
     /**
-     *  @return success
+     *  @return array or null on failure
      *  @since 0.9.18 split out from readBlocklistFile()
      */
-    private void allocate(List<File> files) {
+    private long[] allocate(List<BLFile> files) {
         int maxSize = 0;
-        for (File f : files) {
-            maxSize += getSize(f);
+        for (BLFile blf : files) {
+            maxSize += getSize(blf.file);
         }
         try {
-            _blocklist = new long[maxSize + files.size()];  // extra for wrapsave
+            return new long[maxSize + files.size()];  // extra for wrapsave
         } catch (OutOfMemoryError oom) {
             _log.log(Log.CRIT, "OOM creating the blocklist");
-            disable();
+            return null;
         }
     }
 
@@ -282,10 +358,12 @@ public class Blocklist {
     *
     * Must call allocate() before and merge() after.
     *
+    *  @param blocklist out parameter, entries stored here
     *  @param count current number of entries
     *  @return new number of entries
     */
-    private int readBlocklistFile(File blFile, int count) {
+    private int readBlocklistFile(BLFile blf, long[] blocklist, int count) {
+        File blFile = blf.file;
         if (blFile == null || (!blFile.exists()) || blFile.length() <= 0) {
             if (_log.shouldLog(Log.WARN))
                 _log.warn("Blocklist file not found: " + blFile);
@@ -296,6 +374,7 @@ public class Blocklist {
         int oldcount = count;
         int badcount = 0;
         int peercount = 0;
+        int feedcount = 0;
         long ipcount = 0;
         final boolean isFeedFile = blFile.equals(_blocklistFeedFile);
         BufferedReader br = null;
@@ -319,9 +398,10 @@ public class Blocklist {
                     if (isFeedFile) {
                         // temporary
                         add(ip1);
+                        feedcount++;
                     } else {
                         byte[] ip2 = e.ip2;
-                        store(ip1, ip2, count++);
+                        store(ip1, ip2, blocklist, count++);
                         ipcount += 1 + toInt(ip2) - toInt(ip1); // includes dups, oh well
                     }
                 } else {
@@ -334,9 +414,9 @@ public class Blocklist {
                 _log.error("Error reading the blocklist file", ioe);
             return count;
         } catch (OutOfMemoryError oom) {
-            _blocklist = null;
+            disable();
             _log.log(Log.CRIT, "OOM reading the blocklist");
-            return count;
+            return 0;
         } finally {
             if (br != null) try { br.close(); } catch (IOException ioe) {}
         }
@@ -344,51 +424,57 @@ public class Blocklist {
         if (_wrapSave != null) {
             // the extra record generated in parse() by a line that
             // wrapped around 128.0.0.0
-            store(_wrapSave.ip1, _wrapSave.ip2, count++);
+            store(_wrapSave.ip1, _wrapSave.ip2, blocklist, count++);
             ipcount += 1 + toInt(_wrapSave.ip2) - toInt(_wrapSave.ip1);
             _wrapSave = null;
         }
+        int read = isFeedFile ? feedcount : (count - oldcount);
+        // save to tell the update manager
+        if (read > 0)
+            blf.version = blFile.lastModified();
         if (_log.shouldLog(Log.INFO)) {
             _log.info("Stats for " + blFile);
             _log.info("Removed " + badcount + " bad entries and comment lines");
-            _log.info("Read " + (count - oldcount) + " valid entries from the blocklist " + blFile);
-            _log.info("Blocking " + ipcount + " IPs and " + peercount + " hashes");
+            _log.info("Read " + read + " valid entries from the blocklist " + blFile);
+            _log.info("Blocking " + (isFeedFile ? feedcount : ipcount) + " IPs and " + peercount + " hashes");
             _log.info("Blocklist processing finished, time: " + (_context.clock().now() - start));
         }
         return count;
     }
 
     /**
-     *  @param count valid entries in _blocklist
+     *  @param count valid entries in blocklist before merge
+     *  @return count valid entries in blocklist after merge
      *  @since 0.9.18 split out from readBlocklistFile()
      */
-    private void merge(int count) {
+    private int merge(long[] blocklist, int count) {
         long start = _context.clock().now();
         // This is a standard signed sort, so the entries will be ordered
         // 128.0.0.0 ... 255.255.255.255 0.0.0.0 .... 127.255.255.255
         // But that's ok.
         int removed = 0;
         try {
-            Arrays.sort(_blocklist, 0, count);
-            removed = removeOverlap(_blocklist, count);
+            Arrays.sort(blocklist, 0, count);
+            removed = removeOverlap(blocklist, count);
             if (removed > 0) {
                 // Sort again to remove the dups that were "zeroed" out as 127.255.255.255-255.255.255.255
-                Arrays.sort(_blocklist, 0, count);
+                Arrays.sort(blocklist, 0, count);
                 // sorry, no realloc to save memory, don't want to blow up now
             }
         } catch (OutOfMemoryError oom) {
-            _blocklist = null;
+            disable();
             _log.log(Log.CRIT, "OOM sorting the blocklist");
-            return;
+            return 0;
         }
-        _blocklistSize = count - removed;
+        int blocklistSize = count - removed;
         if (_log.shouldLog(Log.INFO)) {
-            _log.info("Merged Stats");
-            _log.info("Read " + count + " total entries from the blocklists");
-            _log.info("Merged " + removed + " overlapping entries");
-            _log.info("Result is " + _blocklistSize + " entries");
-            _log.info("Blocklist processing finished, time: " + (_context.clock().now() - start));
+            _log.info("Merged Stats:\n" +
+                      "Read " + count + " total entries from the blocklists\n" +
+                      "Merged " + removed + " overlapping entries\n" +
+                      "Result is " + blocklistSize + " entries\n" +
+                      "Blocklist processing finished, time: " + (_context.clock().now() - start));
         }
+        return blocklistSize;
     }
 
     /**
@@ -555,11 +641,11 @@ public class Blocklist {
             for (int next = i + 1; next < count; next++) {
                 if (to < getFrom(blist[next]))
                     break;
-                if (_log.shouldLog(Log.WARN))
-                    _log.warn("Combining entries " + toStr(blist[i]) + " and " + toStr(blist[next]));
+                if (_log.shouldInfo())
+                    _log.info("Combining entries " + toStr(blist[i]) + " and " + toStr(blist[next]));
                 int nextTo = getTo(blist[next]);
                 if (nextTo > to) // else entry next is totally inside entry i
-                    store(getFrom(blist[i]), nextTo, i);
+                    store(getFrom(blist[i]), nextTo, blist, i);
                 blist[next] = Long.MAX_VALUE;  // to be removed with another sort
                 lines++;
                 removed++;
@@ -597,8 +683,8 @@ public class Blocklist {
             rv = add(new BigInteger(1, ip));
         else
             rv = false;
-        if (rv && _log.shouldLog(Log.WARN))
-            _log.warn("Adding IP to blocklist: " + Addresses.toString(ip));
+        if (rv && _log.shouldInfo())
+            _log.info("Adding IP to blocklist: " + Addresses.toString(ip));
     }
 
     /**
@@ -772,26 +858,44 @@ public class Blocklist {
     private boolean isBlocklisted(int ip) {
         if (isOnSingleList(ip))
             return true;
+        if (_countryBlocklist != null) {
+            if (isPermanentlyBlocklisted(ip, _countryBlocklist, _countryBlocklistSize))
+                return true;
+        }
         return isPermanentlyBlocklisted(ip);
     }
 
+    /**
+     * Do a binary search through the in-memory range list which
+     * is a sorted array of longs.
+     * The array is sorted in signed order, but we don't care.
+     * Each long is ((from &lt;&lt; 32) | to)
+     *
+     * Public for console only, not a public API
+     *
+     * @since 0.9.45 split out from above, public since 0.9.48 for console
+     */ 
+    public boolean isPermanentlyBlocklisted(int ip) {
+        return isPermanentlyBlocklisted(ip, _blocklist, _blocklistSize);
+    }
+
     /**
      * Do a binary search through the in-memory range list which
      * is a sorted array of longs.
      * The array is sorted in signed order, but we don't care.
      * Each long is ((from << 32) | to)
      *
-     * @since 0.9.45 split out from above
+     * @since 0.9.48 split out from above
      */ 
-    private boolean isPermanentlyBlocklisted(int ip) {
-        int hi = _blocklistSize - 1;
+    private static boolean isPermanentlyBlocklisted(int ip, long[] blocklist, int blocklistSize) {
+        int hi = blocklistSize - 1;
         if (hi <= 0)
             return false;
         int lo = 0;
         int cur = hi / 2;
 
-        while  (!match(ip, cur)) {
-            if (isHigher(ip, cur))
+        while  (!match(ip, blocklist[cur])) {
+            if (isHigher(ip, blocklist[cur]))
                 lo = cur;
             else
                 hi = cur;
@@ -806,7 +910,7 @@ public class Blocklist {
                 cur = lo + ((hi - lo) / 2);
             }
         }
-        return match(ip, cur);
+        return match(ip, blocklist[cur]);
     }
 
     // Is the IP included in the entry _blocklist[cur] ?
@@ -815,24 +919,32 @@ public class Blocklist {
     }
 
     // Is the IP included in the compressed entry?
-    private boolean match(int ip, long entry) {
+    private static boolean match(int ip, long entry) {
         if (getFrom(entry) > ip)
                 return false;
         return (ip <= getTo(entry));
     }
 
     // Is the IP higher than the entry _blocklist[cur] ?
-    private boolean isHigher(int ip, int cur) {
-        return ip > getFrom(_blocklist[cur]);
+    private static boolean isHigher(int ip, long entry) {
+        return ip > getFrom(entry);
     }
 
     // methods to get and store the from/to values in the array
 
-    private static int getFrom(long entry) {
+    /**
+     *  Public for console only, not a public API
+     *  @since public since 0.9.48
+     */
+    public static int getFrom(long entry) {
         return (int) ((entry >> 32) & 0xffffffff);
     }
 
-    private static int getTo(long entry) {
+    /**
+     *  Public for console only, not a public API
+     *  @since public since 0.9.48
+     */
+    public static int getTo(long entry) {
         return (int) (entry & 0xffffffff);
     }
 
@@ -856,14 +968,14 @@ public class Blocklist {
     /**
      *  IPv4 only
      */
-    private void store(byte ip1[], byte ip2[], int idx) {
-        _blocklist[idx] = toEntry(ip1, ip2);
+    private static void store(byte ip1[], byte ip2[], long[] blocklist, int idx) {
+        blocklist[idx] = toEntry(ip1, ip2);
     }
 
-    private void store(int ip1, int ip2, int idx) {
+    private static void store(int ip1, int ip2, long[] blocklist, int idx) {
         long entry = ((long) ip1) << 32;
         entry |= ((long)ip2) & 0xffffffff;
-        _blocklist[idx] = entry;
+        blocklist[idx] = entry;
     }
 
     private static int toInt(byte ip[]) {
@@ -885,7 +997,11 @@ public class Blocklist {
         return buf.toString();
     }
 
-    private static String toStr(int ip) {
+    /**
+     *  Public for console only, not a public API
+     *  @since public since 0.9.48
+     */
+    public static String toStr(int ip) {
         StringBuilder buf = new StringBuilder(16);
         for (int i = 3; i >= 0; i--) {
             buf.append((ip >> (8*i)) & 0xff);
@@ -1015,151 +1131,82 @@ public class Blocklist {
         // We already banlisted in banlist(peer), that's good enough
     }
 
-    private static final int MAX_DISPLAY = 1000;
-
     /**
-     *  Write directly to the stream so we don't OOM on a huge list.
-     *  Go through each list twice since we store out-of-order.
+     *  Single IPs blocked until restart. Unsorted.
      *
-     *  TODO move to routerconsole, but that would require exposing the _blocklist array.
+     *  Public for console only, not a public API
+     *
+     *  @return a copy, unsorted
+     *  @since 0.9.48
      */
-    public void renderStatusHTML(Writer out) throws IOException {
-        // move to the jsp
-        //out.write("<h2>Banned IPs</h2>");
-        out.write("<table id=\"bannedips\"><tr><td>");
-        out.write("<table id=\"banneduntilrestart\"><tr><th align=\"center\"><b>");
-        out.write(_t("IPs Banned Until Restart"));
-        out.write("</b></th></tr>");
-        Set<Integer> singles = new TreeSet<Integer>();
-        singles.addAll(_singleIPBlocklist);
-        if (!(singles.isEmpty() && _singleIPv6Blocklist.isEmpty())) {
-            if (!singles.isEmpty()) {
-                out.write("<tr id=\"ipv4\" align=\"center\"><td><b>");
-                out.write(_t("IPv4 Addresses"));
-                out.write("</b></td></tr>");
-            }
-            // first 0 - 127
-            for (Integer ii : singles) {
-                 int ip = ii.intValue();
-                 if (ip < 0)
-                     continue;
-                 // don't display if on the permanent blocklist also
-                 if (isPermanentlyBlocklisted(ip))
-                     continue;
-                 out.write("<tr><td align=\"center\">");
-                 out.write(toStr(ip));
-                 out.write("</td></tr>\n");
-            }
-            // then 128 - 255
-            for (Integer ii : singles) {
-                 int ip = ii.intValue();
-                 if (ip >= 0)
-                     break;
-                 // don't display if on the permanent blocklist also
-                 if (isPermanentlyBlocklisted(ip))
-                     continue;
-                 out.write("<tr><td align=\"center\">");
-                 out.write(toStr(ip));
-                 out.write("</td></tr>\n");
-            }
-            // then IPv6
-            if (!_singleIPv6Blocklist.isEmpty()) {
-                out.write("<tr id=\"ipv6\" align=\"center\"><td><b>");
-                out.write(_t("IPv6 Addresses"));
-                out.write("</b></td></tr>");
-                List<BigInteger> s6;
-                synchronized(_singleIPv6Blocklist) {
-                    s6 = new ArrayList<BigInteger>(_singleIPv6Blocklist.keySet());
-                }
-                Collections.sort(s6);
-                for (BigInteger bi : s6) {
-                     out.write("<tr><td align=\"center\">");
-                     out.write(Addresses.toString(toIPBytes(bi)));
-                     out.write("</td></tr>\n");
-                }
-            }
-        } else {
-            out.write("<tr><td><i>");
-            out.write(_t("none"));
-            out.write("</i></td></tr>");
-        }
-        out.write("</table>");
-        out.write("</td><td>");
-        out.write("<table id=\"permabanned\"><tr><th align=\"center\" colspan=\"3\"><b>");
-        out.write(_t("IPs Permanently Banned"));
-        out.write("</b></th></tr>");
-        if (_blocklistSize > 0) {
-            out.write("<tr><td align=\"center\" width=\"49%\"><b>");
-            out.write(_t("From"));
-            out.write("</b></td><td></td><td align=\"center\" width=\"49%\"><b>");
-            out.write(_t("To"));
-            out.write("</b></td></tr>");
-            int max = Math.min(_blocklistSize, MAX_DISPLAY);
-            int displayed = 0;
-            // first 0 - 127
-            for (int i = 0; i < _blocklistSize && displayed < max; i++) {
-                 int from = getFrom(_blocklist[i]);
-                 if (from < 0)
-                     continue;
-                 out.write("<tr><td align=\"center\" width=\"49%\">");
-                 out.write(toStr(from));
-                 out.write("</td>");
-                 int to = getTo(_blocklist[i]);
-                 if (to != from) {
-                     out.write("<td align=\"center\">-</td><td align=\"center\" width=\"49%\">");
-                     out.write(toStr(to));
-                     out.write("</td></tr>\n");
-                 } else
-                     out.write("<td></td><td width=\"49%\">&nbsp;</td></tr>\n");
-                 displayed++;
-            }
-            // then 128 - 255
-            for (int i = 0; i < max && displayed++ < max; i++) {
-                 int from = getFrom(_blocklist[i]);
-                 if (from >= 0)
-                     break;
-                 out.write("<tr><td align=\"center\" width=\"49%\">");
-                 out.write(toStr(from));
-                 out.write("</td>");
-                 int to = getTo(_blocklist[i]);
-                 if (to != from) {
-                     out.write("<td align=\"center\">-</td><td align=\"center\" width=\"49%\">");
-                     out.write(toStr(to));
-                     out.write("</td></tr>\n");
-                 } else
-                     out.write("<td></td><td width=\"49%\">&nbsp;</td></tr>\n");
-            }
-            if (_blocklistSize > MAX_DISPLAY)
-                // very rare, don't bother translating
-                out.write("<tr><th colspan=2>First " + MAX_DISPLAY + " displayed, see the " +
-                          BLOCKLIST_FILE_DEFAULT + " file for the full list</th></tr>");
-        } else {
-            out.write("<tr><td><i>");
-            out.write(_t("none"));
-            out.write("</i></td></tr>");
-        }
-        out.write("</table>");
-        out.write("</td></tr></table>");
-        out.flush();
+    public List<Integer> getTransientIPv4Blocks() {
+        return new ArrayList<Integer>(_singleIPBlocklist);
     }
 
     /**
-     *  Convert a (non-negative) two's complement IP to exactly 16 bytes
-     *  @since IPv6
+     *  Single IPs blocked until restart. Unsorted.
+     *
+     *  Public for console only, not a public API
+     *
+     *  @return a copy, unsorted
+     *  @since 0.9.48
      */
-    private static byte[] toIPBytes(BigInteger bi) {
-        byte[] ba = bi.toByteArray();
-        int len = ba.length;
-        if (len == 16)
-            return ba;
-        byte[] rv = new byte[16];
-        if (len < 16)
-            System.arraycopy(ba, 0, rv, 16 - len, len);
-        else
-            System.arraycopy(ba, len - 16, rv, 0, 16);
+    public List<BigInteger> getTransientIPv6Blocks() {
+        synchronized(_singleIPv6Blocklist) {
+            return new ArrayList<BigInteger>(_singleIPv6Blocklist.keySet());
+        }
+    }
+
+    /**
+     *  IP ranges blocked until restart. Sorted,
+     *  but as signed longs, so 128-255 are first
+     *
+     *  Public for console only, not a public API
+     *
+     *  @param max maximum entries to return
+     *  @return a copy, sorted
+     *  @since 0.9.48
+     */
+    public synchronized long[] getPermanentBlocks(int max) {
+        long[] rv;
+        if (_blocklistSize <= max) {
+            rv = new long[_blocklistSize];
+            System.arraycopy(_blocklist, 0, rv, 0, _blocklistSize);
+        } else {
+            // skip ahead to the positive entries
+            int i = 0;
+            for (; i < _blocklistSize; i++) {
+                int from = Blocklist.getFrom(_blocklist[i]);
+                if (from >= 0)
+                    break;
+            }
+            int sz = Math.min(_blocklistSize - i, max);
+            rv = new long[sz];
+            System.arraycopy(_blocklist, i, rv, 0, sz);
+        }
         return rv;
     }
 
+    /**
+     *  Size of permanent blocklist
+     *
+     *  Public for console only, not a public API
+     *
+     *  @since 0.9.48
+     */
+    public synchronized int getBlocklistSize() {
+        return _blocklistSize;
+    }
+
+    /**
+     *  Does nothing, moved to console ConfigPeerHelper
+     *
+     *  @deprecated
+     */
+    @Deprecated
+    public void renderStatusHTML(Writer out) throws IOException {
+    }
+
     /**
      *  Mark a string for extraction by xgettext and translation.
      *  Use this only in static initializers.
@@ -1170,13 +1217,6 @@ public class Blocklist {
         return s;
     }
 
-    private static final String BUNDLE_NAME = "net.i2p.router.web.messages";
-
-    /** translate */
-    private String _t(String key) {
-        return Translate.getString(key, _context, BUNDLE_NAME);
-    }
-
 /****
     public static void main(String args[]) throws Exception {
         Blocklist b = new Blocklist(new Router().getContext());
