@@ -16,11 +16,14 @@ import java.util.Set;
 import java.util.SortedMap;
 import java.util.TreeMap;
 
+import net.i2p.I2PAppContext;
 import net.i2p.data.Hash;
 import net.i2p.data.router.RouterAddress;
 import net.i2p.data.router.RouterInfo;
 import net.i2p.router.transport.Transport;
 import net.i2p.router.transport.crypto.DHSessionKeyBuilder;
+import net.i2p.router.transport.crypto.X25519KeyFactory;
+import net.i2p.util.Translate;
 
 /**
  * Manages the communication subsystem between peers, including connections, 
@@ -28,6 +31,10 @@ import net.i2p.router.transport.crypto.DHSessionKeyBuilder;
  *
  */ 
 public abstract class CommSystemFacade implements Service {
+
+    /** @since 0.9.45 */
+    protected static final String ROUTER_BUNDLE_NAME = "net.i2p.router.util.messages";
+
     public abstract void processMessage(OutNetMessage msg);
     
     public void renderStatusHTML(Writer out, String urlBase, int sortFlags) throws IOException { }
@@ -83,6 +90,14 @@ public abstract class CommSystemFacade implements Service {
      */
     public Status getStatus() { return Status.OK; }
 
+    /**
+     * getStatus().toStatusString(), translated if available.
+     * @since 0.9.45
+     */
+    public String getLocalizedStatusString() {
+        return getStatus().toStatusString();
+    }
+
     /**
      * @deprecated unused
      */
@@ -184,6 +199,12 @@ public abstract class CommSystemFacade implements Service {
      */
     public DHSessionKeyBuilder.Factory getDHFactory() { return null; }
 
+    /**
+     *  Factory for making X25519 key pairs.
+     *  @since 0.9.46
+     */
+    public X25519KeyFactory getXDHFactory() { return null; }
+
     /**
      *  Router must call after netdb is initialized
      *  @since 0.9.41
@@ -410,8 +431,6 @@ public abstract class CommSystemFacade implements Service {
 
                         case DIFFERENT:
                         case REJECT_UNSOLICITED:
-                            return newStatus;
-
                         case IPV4_DISABLED_IPV6_FIREWALLED:
                             return IPV4_OK_IPV6_FIREWALLED;
 
@@ -483,12 +502,12 @@ public abstract class CommSystemFacade implements Service {
                         case IPV4_SNAT_IPV6_OK:
                             return IPV4_FIREWALLED_IPV6_OK;
 
+                        case REJECT_UNSOLICITED:
                         case IPV4_OK_IPV6_FIREWALLED:
                         case IPV4_DISABLED_IPV6_FIREWALLED:
                             return REJECT_UNSOLICITED;
 
                         case DIFFERENT:
-                        case REJECT_UNSOLICITED:
                             return newStatus;
 
                         // cases where we already knew the IPv6 state only
@@ -637,6 +656,14 @@ public abstract class CommSystemFacade implements Service {
             return status;
         }
 
+        /**
+         * toStatusString(), translated if available.
+         * @since 0.9.45
+         */
+        public String toLocalizedStatusString(I2PAppContext ctx) {
+            return Translate.getString(status, ctx, ROUTER_BUNDLE_NAME);
+        }
+
         @Override
         public String toString() {
             return super.toString() + " (" + code + "; " + status + ')';
