@@ -44,7 +44,7 @@ public class InNetMessagePool implements Service {
     private SharedShortCircuitGatewayJob _shortCircuitGatewayJob;
 
     private boolean _alive;
-    private boolean _dispatchThreaded;
+    private final boolean _dispatchThreaded;
     
     /** Make this >= the max I2NP message type number (currently 24) */
     private static final int MAX_I2NP_MESSAGE_TYPE = 31;
@@ -56,20 +56,40 @@ public class InNetMessagePool implements Service {
      * if this is set to false, the messages will be queued up in the jobQueue,
      * using the jobQueue's single thread.
      *
+     * There's three possible cases that might work:
+     * DISPATCH_DIRECT=true (default, unqueued)
+     * DISPATCH_DIRECT=false with router.dispatchThreaded=false (job queue)
+     * DISPATCH_DIRECT=false with router.dispatchThreaded=true (INMP queue)
+     *
+     * The fourth case,
+     * DISPATCH_DIRECT=true with router.dispatchThreaded=true
+     * will never work, is now prevented below.
+     *
+     * Both must be configured before starting.
+     * Changing defaults not recommended, non-default code to be removed.
+     * Ref: Ticket 2688
      */
-    public static final String PROP_DISPATCH_THREADED = "router.dispatchThreaded";
-    public static final boolean DEFAULT_DISPATCH_THREADED = false;
+    private static final String PROP_DISPATCH_THREADED = "router.dispatchThreaded";
+    private static final boolean DEFAULT_DISPATCH_THREADED = false;
     /**
      * If we aren't doing threaded dispatch for tunnel messages, should we
      * call the actual dispatch() method inline (on the same thread which
      * called add())?  If false, we queue it up in a shared short circuit
      * job.
+     *
+     * false = job queue
+     * true = INMP queue
      */
-    private static final boolean DISPATCH_DIRECT = true;
+    private static final String PROP_DISPATCH_DIRECT = "router.dispatchDirect";
+    private static final boolean DEFAULT_DISPATCH_DIRECT = true;
+    private final boolean DISPATCH_DIRECT;
     
     public InNetMessagePool(RouterContext context) {
         _context = context;
         _handlerJobBuilders = new HandlerJobBuilder[MAX_I2NP_MESSAGE_TYPE + 1];
+        _dispatchThreaded = _context.getProperty(PROP_DISPATCH_THREADED, DEFAULT_DISPATCH_THREADED);
+        // must be false if threaded is true
+        DISPATCH_DIRECT = !_dispatchThreaded && _context.getProperty(PROP_DISPATCH_DIRECT, DEFAULT_DISPATCH_DIRECT);
         if (DISPATCH_DIRECT) {
             // keep the compiler happy since they are final
             _pendingDataMessages = null;
@@ -126,10 +146,13 @@ public class InNetMessagePool implements Service {
      *         (was queue length, long ago)
      */
     public int add(I2NPMessage messageBody, RouterIdentity fromRouter, Hash fromRouterHash) {
+        final MessageHistory history = _context.messageHistory();
+        final boolean doHistory = history.getDoLog();
+
         long exp = messageBody.getMessageExpiration();
         
-        if (_log.shouldLog(Log.INFO))
-                _log.info("Rcvd" 
+        if (_log.shouldDebug())
+                _log.debug("Rcvd" 
                           + " ID " + messageBody.getUniqueId()
                           + " exp. " + new Date(exp)
                           + " type " + messageBody.getClass().getSimpleName());
@@ -160,15 +183,13 @@ public class InNetMessagePool implements Service {
             _context.statManager().addRateData("inNetPool.dropped", 1);
             // FIXME not necessarily a duplicate, could be expired too long ago / too far in future
             _context.statManager().addRateData("inNetPool.duplicate", 1);
-            _context.messageHistory().droppedOtherMessage(messageBody, (fromRouter != null ? fromRouter.calculateHash() : fromRouterHash));
-            _context.messageHistory().messageProcessingError(messageBody.getUniqueId(), 
+            if (doHistory) {
+                history.droppedOtherMessage(messageBody, (fromRouter != null ? fromRouter.calculateHash() : fromRouterHash));
+                history.messageProcessingError(messageBody.getUniqueId(), 
                                                                 messageBody.getClass().getSimpleName(), 
                                                                 "Duplicate/expired");
+            }
             return -1;
-        } else {
-            if (_log.shouldLog(Log.DEBUG))
-                _log.debug("Message received [" + messageBody.getUniqueId() 
-                           + " expiring on " + exp + "] is NOT a duplicate or exipired");
         }
 
         boolean jobFound = false;
@@ -199,12 +220,11 @@ public class InNetMessagePool implements Service {
                                                 fromRouterHash);
                     if (job != null) {
                         _context.jobQueue().addJob(job);
-                        jobFound = true;
                     } else {
                         // ok, we may not have *found* a job, per se, but we could have, the
                         // job may have just executed inline
-                        jobFound = true;
                     }
+                    jobFound = true;
                 }
             }
             break;
@@ -217,7 +237,8 @@ public class InNetMessagePool implements Service {
                 // not handled as a reply
                 if (!jobFound) { 
                     // was not handled via HandlerJobBuilder
-                    _context.messageHistory().droppedOtherMessage(messageBody, (fromRouter != null ? fromRouter.calculateHash() : fromRouterHash));
+                    if (doHistory)
+                        history.droppedOtherMessage(messageBody, (fromRouter != null ? fromRouter.calculateHash() : fromRouterHash));
 
                     switch (type) { 
                       case DeliveryStatusMessage.MESSAGE_TYPE:
@@ -233,9 +254,12 @@ public class InNetMessagePool implements Service {
                         break;
 
                       case DatabaseSearchReplyMessage.MESSAGE_TYPE:
-                        if (_log.shouldLog(Log.INFO))
-                            _log.info("Dropping slow db lookup response: " + messageBody);
-                        _context.statManager().addRateData("inNetPool.droppedDbLookupResponseMessage", 1);
+                        // This is normal.
+                        // The three netdb selectors,
+                        // FloodOnlyLookupSelector, IterativeLookupSelector, and SearchMessageSelector
+                        // never return true from isMatch() for a DSRM.
+                        // IterativeLookupSelector.isMatch() queues a new IterativeLookupJob
+                        // to fetch the responses.
                         break;
 
                       case DatabaseLookupMessage.MESSAGE_TYPE:
@@ -253,19 +277,23 @@ public class InNetMessagePool implements Service {
                         break;
                     }  // switch
                 } else {
-                    String mtype = messageBody.getClass().getName();
-                    _context.messageHistory().receiveMessage(mtype, messageBody.getUniqueId(), 
+                    if (doHistory) {
+                        String mtype = messageBody.getClass().getName();
+                        history.receiveMessage(mtype, messageBody.getUniqueId(), 
                                                              messageBody.getMessageExpiration(), 
                                                              fromRouterHash, true);	
+                    }
                     return 0; // no queue
                 }
             }
         }
 
-        String mtype = messageBody.getClass().getName();
-        _context.messageHistory().receiveMessage(mtype, messageBody.getUniqueId(), 
+        if (doHistory) {
+            String mtype = messageBody.getClass().getName();
+            history.receiveMessage(mtype, messageBody.getUniqueId(), 
                                                  messageBody.getMessageExpiration(), 
                                                  fromRouterHash, true);	
+        }
         return 0; // no queue
     }
     
@@ -313,8 +341,8 @@ public class InNetMessagePool implements Service {
     }
 
     private void doShortCircuitTunnelGateway(I2NPMessage messageBody) {
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Shortcut dispatch tunnelGateway message " + messageBody);
+        //if (_log.shouldLog(Log.DEBUG))
+        //    _log.debug("Shortcut dispatch tunnelGateway message " + messageBody);
         _context.tunnelDispatcher().dispatch((TunnelGatewayMessage)messageBody);
     }
     
@@ -333,8 +361,8 @@ public class InNetMessagePool implements Service {
         }
     }
     private void doShortCircuitTunnelData(I2NPMessage messageBody, Hash from) {
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Shortcut dispatch tunnelData message " + messageBody);
+        //if (_log.shouldLog(Log.DEBUG))
+        //    _log.debug("Shortcut dispatch tunnelData message " + messageBody);
         _context.tunnelDispatcher().dispatch((TunnelDataMessage)messageBody, from);
     }
     
@@ -362,11 +390,6 @@ public class InNetMessagePool implements Service {
     /** does nothing since we aren't threaded */
     public synchronized void startup() {
         _alive = true;
-        _dispatchThreaded = DEFAULT_DISPATCH_THREADED;
-        String threadedStr = _context.getProperty(PROP_DISPATCH_THREADED);
-        if (threadedStr != null) {
-            _dispatchThreaded = Boolean.parseBoolean(threadedStr);
-        }
         if (_dispatchThreaded) {
             _context.statManager().createRateStat("pool.dispatchDataTime", "How long a tunnel dispatch takes", "Tunnels", new long[] { 10*60*1000l, 60*60*1000l, 24*60*60*1000l });
             _context.statManager().createRateStat("pool.dispatchGatewayTime", "How long a tunnel gateway dispatch takes", "Tunnels", new long[] { 10*60*1000l, 60*60*1000l, 24*60*60*1000l });
