@@ -71,7 +71,7 @@ class MessageReceivedJob extends JobImpl {
             }
             return true;
         } catch (I2CPMessageException ime) {
-            String msg = "Error sending data to client " + _runner.getDestHash();
+            String msg = "Error sending data to client " + _runner.getDestHash().toBase32();
             if (_log.shouldWarn())
                 _log.warn(msg, ime);
             else
@@ -94,7 +94,7 @@ class MessageReceivedJob extends JobImpl {
         SessionId sid = _runner.getSessionId(_toDest.calculateHash());
         if (sid == null) {
             if (_log.shouldLog(Log.WARN))
-                _log.warn("No session for " + _toDest.calculateHash());
+                _log.warn("No session for " + _toDest.toBase32());
             return;
         }
         msg.setSessionId(sid.getSessionId());
@@ -115,7 +115,7 @@ class MessageReceivedJob extends JobImpl {
         SessionId sid = _runner.getSessionId(_toDest.calculateHash());
         if (sid == null) {
             if (_log.shouldLog(Log.WARN))
-                _log.warn("No session for " + _toDest.calculateHash());
+                _log.warn("No session for " + _toDest.toBase32());
             return;
         }
         msg.setSessionId(sid.getSessionId());
