@@ -103,14 +103,15 @@ class FloodfillVerifyStoreJob extends JobImpl {
             return;
         }        
 
+        final RouterContext ctx = getContext();
         boolean isInboundExploratory;
         TunnelInfo replyTunnelInfo;
-        if (_isRouterInfo || getContext().keyRing().get(_key) != null ||
+        if (_isRouterInfo || ctx.keyRing().get(_key) != null ||
             _type == DatabaseEntry.KEY_TYPE_META_LS2) {
-            replyTunnelInfo = getContext().tunnelManager().selectInboundExploratoryTunnel(_target);
+            replyTunnelInfo = ctx.tunnelManager().selectInboundExploratoryTunnel(_target);
             isInboundExploratory = true;
         } else {
-            replyTunnelInfo = getContext().tunnelManager().selectInboundTunnel(_client, _target);
+            replyTunnelInfo = ctx.tunnelManager().selectInboundTunnel(_client, _target);
             isInboundExploratory = false;
         }
         if (replyTunnelInfo == null) {
@@ -125,11 +126,11 @@ class FloodfillVerifyStoreJob extends JobImpl {
         // to avoid association by the exploratory tunnel OBEP.
         // Unless it is an encrypted leaseset.
         TunnelInfo outTunnel;
-        if (_isRouterInfo || getContext().keyRing().get(_key) != null ||
+        if (_isRouterInfo || ctx.keyRing().get(_key) != null ||
             _type == DatabaseEntry.KEY_TYPE_META_LS2) {
-            outTunnel = getContext().tunnelManager().selectOutboundExploratoryTunnel(_target);
+            outTunnel = ctx.tunnelManager().selectOutboundExploratoryTunnel(_target);
         } else {
-            outTunnel = getContext().tunnelManager().selectOutboundTunnel(_client, _target);
+            outTunnel = ctx.tunnelManager().selectOutboundTunnel(_client, _target);
         }
         if (outTunnel == null) {
             if (_log.shouldLog(Log.WARN))
@@ -146,23 +147,41 @@ class FloodfillVerifyStoreJob extends JobImpl {
             _facade.verifyFinished(_key);
             return;
         }
+        EncType type = peer.getIdentity().getPublicKey().getType();
         boolean supportsElGamal = true;
         boolean supportsRatchet = false;
         if (DatabaseLookupMessage.supportsEncryptedReplies(peer)) {
             // register the session with the right SKM
             MessageWrapper.OneTimeSession sess;
             if (isInboundExploratory) {
-                sess = MessageWrapper.generateSession(getContext(), VERIFY_TIMEOUT);
+                EncType ourType = ctx.keyManager().getPublicKey().getType();
+                supportsRatchet = ourType == EncType.ECIES_X25519 &&
+                                  type == EncType.ECIES_X25519;
+                supportsElGamal = ourType == EncType.ELGAMAL_2048 &&
+                                  type == EncType.ELGAMAL_2048;
+                if (supportsElGamal || supportsRatchet) {
+                    sess = MessageWrapper.generateSession(ctx, ctx.sessionKeyManager(), VERIFY_TIMEOUT, !supportsRatchet);
+                } else {
+                    // We don't have a compatible way to get a reply,
+                    // skip it for now.
+                     if (_log.shouldWarn())
+                         _log.warn("Skipping store verify for incompatible router " + peer);
+                    _facade.verifyFinished(_key);
+                    return;
+                }
             } else {
-                LeaseSetKeys lsk = getContext().keyManager().getKeys(_client);
+                LeaseSetKeys lsk = ctx.keyManager().getKeys(_client);
+                // an ElG router supports ratchet replies
                 supportsRatchet = lsk != null &&
                                   lsk.isSupported(EncType.ECIES_X25519) &&
                                   DatabaseLookupMessage.supportsRatchetReplies(peer);
+                // but an ECIES router does not supports ElGamal requests
                 supportsElGamal = lsk != null &&
-                                  lsk.isSupported(EncType.ELGAMAL_2048);
+                                  lsk.isSupported(EncType.ELGAMAL_2048) &&
+                                  type == EncType.ELGAMAL_2048;
                 if (supportsElGamal || supportsRatchet) {
                     // garlic encrypt
-                    sess = MessageWrapper.generateSession(getContext(), _client, VERIFY_TIMEOUT, !supportsRatchet);
+                    sess = MessageWrapper.generateSession(ctx, _client, VERIFY_TIMEOUT, !supportsRatchet);
                     if (sess == null) {
                          if (_log.shouldLog(Log.WARN))
                              _log.warn("No SKM to reply to");
@@ -180,11 +199,11 @@ class FloodfillVerifyStoreJob extends JobImpl {
             }
             if (sess.tag != null) {
                 if (_log.shouldInfo())
-                    _log.info(getJobId() + ": Requesting AES reply from " + peer + ' ' + sess.key + ' ' + sess.tag);
+                    _log.info(getJobId() + ": Requesting AES reply from " + _target + " with: " + sess.key + ' ' + sess.tag);
                 lookup.setReplySession(sess.key, sess.tag);
             } else {
                 if (_log.shouldInfo())
-                    _log.info(getJobId() + ": Requesting AEAD reply from " + peer + ' ' + sess.key + ' ' + sess.rtag);
+                    _log.info(getJobId() + ": Requesting AEAD reply from " + _target + " with: " + sess.key + ' ' + sess.rtag);
                 lookup.setReplySession(sess.key, sess.rtag);
             }
         }
@@ -195,7 +214,7 @@ class FloodfillVerifyStoreJob extends JobImpl {
                 fromKey = null;
             else
                 fromKey = _client;
-            _wrappedMessage = MessageWrapper.wrap(getContext(), lookup, fromKey, peer);
+            _wrappedMessage = MessageWrapper.wrap(ctx, lookup, fromKey, peer);
             if (_wrappedMessage == null) {
                  if (_log.shouldLog(Log.WARN))
                     _log.warn("Fail Garlic encrypting");
@@ -205,7 +224,8 @@ class FloodfillVerifyStoreJob extends JobImpl {
             sent = _wrappedMessage.getMessage();
         } else {
             // force full ElG for ECIES fromkey
-            sent = MessageWrapper.wrap(getContext(), lookup, peer);
+            // or forces ECIES for ECIES peer
+            sent = MessageWrapper.wrap(ctx, lookup, peer);
             if (sent == null) {
                  if (_log.shouldLog(Log.WARN))
                     _log.warn("Fail Garlic encrypting");
@@ -216,12 +236,12 @@ class FloodfillVerifyStoreJob extends JobImpl {
 
         if (_log.shouldLog(Log.INFO))
             _log.info(getJobId() + ": Starting verify (stored " + _key + " to " + _sentTo + "), asking " + _target);
-        _sendTime = getContext().clock().now();
+        _sendTime = ctx.clock().now();
         _expiration = _sendTime + VERIFY_TIMEOUT;
-        getContext().messageRegistry().registerPending(new VerifyReplySelector(),
+        ctx.messageRegistry().registerPending(new VerifyReplySelector(),
                                                        new VerifyReplyJob(getContext()),
                                                        new VerifyTimeoutJob(getContext()));
-        getContext().tunnelDispatcher().dispatchOutbound(sent, outTunnel.getSendTunnelId(0), _target);
+        ctx.tunnelDispatcher().dispatchOutbound(sent, outTunnel.getSendTunnelId(0), _target);
     }
     
     /**
