@@ -116,16 +116,16 @@ class HandleFloodfillDatabaseStoreMessageJob extends JobImpl {
                 } else if (match.getEarliestLeaseDate() < ls.getEarliestLeaseDate()) {
                     wasNew = true;
                     // If it is in our keyspace and we are talking to it
-                    if (match.getReceivedAsPublished())
-                        ls.setReceivedAsPublished(true);
+                    //if (match.getReceivedAsPublished())
+                    //    ls.setReceivedAsPublished(true);
                 } else if (type != DatabaseEntry.KEY_TYPE_LEASESET &&
                            match.getType() != DatabaseEntry.KEY_TYPE_LEASESET) {
                     LeaseSet2 ls2 = (LeaseSet2) ls;
                     LeaseSet2 match2 = (LeaseSet2) match;
                     if (match2.getPublished() < ls2.getPublished()) {
                         wasNew = true;
-                        if (match.getReceivedAsPublished())
-                            ls.setReceivedAsPublished(true);
+                        //if (match.getReceivedAsPublished())
+                        //    ls.setReceivedAsPublished(true);
                     } else {
                         wasNew = false;
                     }
