@@ -44,15 +44,17 @@ class IterativeLookupSelector implements MessageSelector {
             // is it worth making sure the reply came in on the right tunnel?
             if (_search.getKey().equals(dsm.getKey())) {
                 _matchFound = true;
+                if (_log.shouldDebug())
+                    _log.debug(_search.getJobId() + ": DSM match " + this);
                 return true;
             }
         } else if (type == DatabaseSearchReplyMessage.MESSAGE_TYPE) {
             DatabaseSearchReplyMessage dsrm = (DatabaseSearchReplyMessage)message;
             if (_search.getKey().equals(dsrm.getSearchKey())) {
                 // Got a netDb reply pointing us at other floodfills...
-                if (_log.shouldLog(Log.INFO)) {
+                if (_log.shouldDebug()) {
                     Hash from = dsrm.getFromHash();
-                    _log.info(_search.getJobId() + ": Processing DSRM via IterativeLookupJob, apparently from " + from);
+                    _log.debug(_search.getJobId() + ": Processing DSRM via IterativeLookupJob, apparently from " + from + ' ' + this);
                 }
 
                 // was inline, now in IterativeLookupJob due to deadlocks
