@@ -40,6 +40,7 @@ import net.i2p.util.I2PThread;
 import net.i2p.util.Log;
 import net.i2p.util.SecureDirectory;
 import net.i2p.util.SecureFileOutputStream;
+import net.i2p.util.SystemVersion;
 
 /**
  * Write out keys to disk when we get them and periodically read ones we don't know
@@ -342,6 +343,7 @@ public class PersistentDataStore extends TransientDataStore {
     private class ReadJob extends JobImpl {
         private volatile long _lastModified;
         private volatile long _lastReseed;
+        private volatile boolean _setNetDbReady;
         private static final int MIN_ROUTERS = KademliaNetworkDatabaseFacade.MIN_RESEED;
         private static final long MIN_RESEED_INTERVAL = 90*60*1000;
 
@@ -434,10 +436,24 @@ public class PersistentDataStore extends TransientDataStore {
                     }
                 }
                 Collections.shuffle(toRead, _context.random());
+                int i = 0;
                 for (File file : toRead) {
                     Hash key = getRouterInfoHash(file.getName());
-                    if (key != null && !isKnown(key))
+                    if (key != null && !isKnown(key)) {
                         (new ReadRouterJob(file, key)).runJob();
+                        if (i++ == 150 && SystemVersion.isSlow() && !_initialized) {
+                            // Can take 2 minutes to load them all on Android,
+                            // after we have already built expl. tunnels.
+                            // This is enough to let i2ptunnel get started.
+                            // Do not set _initialized yet so we don't start rescanning.
+                            _setNetDbReady = true;
+                            _context.router().setNetDbReady();
+                        } else if (i == 500 && !_setNetDbReady) {
+                            // do this for faster systems also at 500
+                            _setNetDbReady = true;
+                            _context.router().setNetDbReady();
+                        }
+                    }
                 }
             }
             
@@ -447,6 +463,7 @@ public class PersistentDataStore extends TransientDataStore {
                     _lastReseed = _context.clock().now();
                     // checkReseed will call wakeup() when done and we will run again
                 } else {
+                    _setNetDbReady = true;
                     _context.router().setNetDbReady();
                 }
             } else if (_lastReseed < _context.clock().now() - MIN_RESEED_INTERVAL) {
@@ -456,7 +473,19 @@ public class PersistentDataStore extends TransientDataStore {
                         _lastReseed = _context.clock().now();
                         // checkReseed will call wakeup() when done and we will run again
                 } else {
-                    _context.router().setNetDbReady();
+                    if (!_setNetDbReady) {
+                        _setNetDbReady = true;
+                        _context.router().setNetDbReady();
+                    }
+                }
+            } else {
+                // second time through, reseed called wakeup()
+                if (!_setNetDbReady) {
+                    int count = Math.min(routerCount, size());
+                    if (count >= MIN_ROUTERS) {
+                        _setNetDbReady = true;
+                        _context.router().setNetDbReady();
+                    }
                 }
             }
         }
