@@ -7,6 +7,7 @@ import java.util.HashSet;
 import java.util.Set;
 
 import net.i2p.data.Hash;
+import net.i2p.router.CommSystemFacade;
 import net.i2p.router.RouterContext;
 import net.i2p.stat.RateStat;
 import net.i2p.util.Log;
@@ -58,7 +59,6 @@ public class PeerProfile {
     private float _speedValue;
     private float _capacityValue;
     private float _integrationValue;
-    private boolean _isFailing;
     // new calculation values, to be updated
     // floats to save some space
     private float _speedValueNew;
@@ -86,10 +86,16 @@ public class PeerProfile {
     private final float _peakTunnelThroughput[] = new float[THROUGHPUT_COUNT];
     /** total number of bytes pushed through a single tunnel in a 1 minute period */
     private final float _peakTunnel1mThroughput[] = new float[THROUGHPUT_COUNT];
-    /** once a day, on average, cut the measured throughtput values in half */
-    /** let's try once an hour times 3/4 */
-    private static final int DROP_PERIOD_MINUTES = 60;
-    private static final float DEGRADE_FACTOR = 0.75f;
+    /** periodically cut the measured throughput values */
+    private static final int DEGRADES_PER_DAY = 4;
+    // one in this many times, ~= 61
+    private static final int DEGRADE_PROBABILITY = PeerManager.REORGANIZES_PER_DAY / DEGRADES_PER_DAY;
+    private static final double TOTAL_DEGRADE_PER_DAY = 0.5d;
+    // the goal is to cut an unchanged profile in half in 24 hours.
+    // x**4 = .5; x = 4th root of .5,  x = .5**(1/4), x ~= 0.84
+    private static final float DEGRADE_FACTOR = (float) Math.pow(TOTAL_DEGRADE_PER_DAY, 1.0d / DEGRADES_PER_DAY);
+    //static { System.out.println("Degrade factor is " + DEGRADE_FACTOR); }
+
     private long _lastCoalesceDate = System.currentTimeMillis();
     
     /**
@@ -100,7 +106,7 @@ public class PeerProfile {
     private static final Set<String> _bigCountries = new HashSet<String>();
 
     static {
-        String[] big = new String[] { "fr", "de", "ru", "ua", "us" };
+        String[] big = new String[] { "fr", "de", "ru", "au", "us", "ca", "gb", "jp", "nl" };
         _bigCountries.addAll(Arrays.asList(big));
     }
 
@@ -162,21 +168,33 @@ public class PeerProfile {
     
     /** @since 0.8.11 */
     boolean isEstablished() {
-        return _context.commSystem().isEstablished(_peer);
+        // null for tests
+        CommSystemFacade cs = _context.commSystem();
+        if (cs == null)
+            return false;
+        return cs.isEstablished(_peer);
     }
 
     /** @since 0.8.11 */
     boolean wasUnreachable() {
-        return _context.commSystem().wasUnreachable(_peer);
+        // null for tests
+        CommSystemFacade cs = _context.commSystem();
+        if (cs == null)
+            return false;
+        return cs.wasUnreachable(_peer);
     }
 
     /** @since 0.8.11 */
     boolean isSameCountry() {
-        String us = _context.commSystem().getOurCountry();
+        // null for tests
+        CommSystemFacade cs = _context.commSystem();
+        if (cs == null)
+            return false;
+        String us = cs.getOurCountry();
         return us != null &&
                (_bigCountries.contains(us) ||
                 _context.getProperty(CapacityCalculator.PROP_COUNTRY_BONUS) != null) &&
-               us.equals(_context.commSystem().getCountry(_peer));
+               us.equals(cs.getCountry(_peer));
     }
 
     /**
@@ -214,7 +232,7 @@ public class PeerProfile {
         long before = _context.clock().now() - period;
         return getLastHeardFrom() < before ||
                getLastSendSuccessful() < before ||
-             _context.commSystem().isEstablished(_peer);
+               isEstablished();
     }
     
     
@@ -336,7 +354,7 @@ public class PeerProfile {
      * is this peer actively failing (aka not worth touching)?
      * deprecated - unused - always false
      */
-    public boolean getIsFailing() { return _isFailing; }
+    public boolean getIsFailing() { return false; }
 
     public float getTunnelTestTimeAverage() { return _tunnelTestResponseTimeAvg; }
     void setTunnelTestTimeAverage(float avg) { _tunnelTestResponseTimeAvg = avg; }
@@ -363,11 +381,18 @@ public class PeerProfile {
         rv /= (60 * 1024 * THROUGHPUT_COUNT);
         return rv;
     }
-    public void setPeakThroughputKBps(float kBps) {
-        _peakThroughput[0] = kBps*60*1024;
-        //for (int i = 0; i < THROUGHPUT_COUNT; i++)
-        //    _peakThroughput[i] = kBps*60;
+
+    /**
+     *  Only for restoration from persisted profile.
+     */
+    void setPeakThroughputKBps(float kBps) {
+        // Set all so the average remains the same
+        float speed = kBps * (60 * 1024);
+        for (int i = 0; i < THROUGHPUT_COUNT; i++) {
+            _peakThroughput[i] = speed;
+        }
     }
+
     void dataPushed(int size) { _peakThroughputCurrentTotal += size; }
     
     /** the tunnel pushed that much data in its lifetime */
@@ -393,8 +418,16 @@ public class PeerProfile {
         rv /= (10 * 60 * 1024 * THROUGHPUT_COUNT);
         return rv;
     }
-    public void setPeakTunnelThroughputKBps(float kBps) {
-        _peakTunnelThroughput[0] = kBps * (60 * 10 * 1024);
+
+    /**
+     *  Only for restoration from persisted profile.
+     */
+    void setPeakTunnelThroughputKBps(float kBps) {
+        // Set all so the average remains the same
+        float speed = kBps * (60 * 10 * 1024);
+        for (int i = 0; i < THROUGHPUT_COUNT; i++) {
+            _peakTunnelThroughput[i] = speed;
+        }
     }
     
     /** the tunnel pushed that much data in a 1 minute period */
@@ -422,10 +455,13 @@ public class PeerProfile {
             }
         }
     }
+
     /**
+     * This is the speed value
+     *
      * @return the average of the three fastest one-minute data transfers, on a per-tunnel basis,
      *         through this peer. Ever. Except that the peak values are cut in half
-     *         once a day by coalesceThroughput(). This seems way too seldom.
+     *         periodically by coalesceThroughput().
      */
     public float getPeakTunnel1mThroughputKBps() { 
         float rv = 0;
@@ -434,8 +470,16 @@ public class PeerProfile {
         rv /= (60 * 1024 * THROUGHPUT_COUNT);
         return rv;
     }
-    public void setPeakTunnel1mThroughputKBps(float kBps) {
-        _peakTunnel1mThroughput[0] = kBps*60*1024;
+
+    /**
+     *  Only for restoration from persisted profile.
+     */
+    void setPeakTunnel1mThroughputKBps(float kBps) {
+        // Set all so the average remains the same
+        float speed = kBps * (60 * 1024);
+        for (int i = 0; i < THROUGHPUT_COUNT; i++) {
+            _peakTunnel1mThroughput[i] = speed;
+        }
     }
     
     /**
@@ -501,10 +545,12 @@ public class PeerProfile {
         _expandedDB = true;
     }
 
-    private void coalesceThroughput() {
+    private void coalesceThroughput(boolean decay) {
         long now = System.currentTimeMillis();
         long measuredPeriod = now - _lastCoalesceDate;
         if (measuredPeriod >= 60*1000) {
+            // so we don't call random() twice
+            boolean shouldDecay =  decay && _context.random().nextInt(DEGRADE_PROBABILITY) <= 0;
             long tot = _peakThroughputCurrentTotal;
             float lowPeak = _peakThroughput[THROUGHPUT_COUNT-1];
             if (tot > lowPeak) {
@@ -517,7 +563,7 @@ public class PeerProfile {
                     }
                 }
             } else {
-                if (_context.random().nextInt(DROP_PERIOD_MINUTES*2) <= 0) {
+                if (shouldDecay) {
                     for (int i = 0; i < THROUGHPUT_COUNT; i++)
                         _peakThroughput[i] *= DEGRADE_FACTOR;
                 }
@@ -525,7 +571,7 @@ public class PeerProfile {
             
             // we degrade the tunnel throughput here too, regardless of the current
             // activity
-            if (_context.random().nextInt(DROP_PERIOD_MINUTES*2) <= 0) {
+            if (shouldDecay) {
                 for (int i = 0; i < THROUGHPUT_COUNT; i++) {
                     _peakTunnelThroughput[i] *= DEGRADE_FACTOR;
                     _peakTunnel1mThroughput[i] *= DEGRADE_FACTOR;
@@ -536,22 +582,24 @@ public class PeerProfile {
         }
     }
     
-    /** update the stats and rates (this should be called once a minute) */
-    public void coalesceStats() {
+    /**
+     *  Update the stats and rates. This is only called by addProfile()
+     */
+    void coalesceStats() {
         if (!_expanded) return;
 
-        coalesceOnly();
+        coalesceOnly(false);
         updateValues();
         
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Coalesced: speed [" + _speedValue + "] capacity [" + _capacityValue + "] integration [" + _integrationValue + "] failing? [" + _isFailing + "]");
+            _log.debug("Coalesced: speed [" + _speedValue + "] capacity [" + _capacityValue + "] integration [" + _integrationValue + ']');
     }
     
     /**
      *  Caller must next call updateValues()
      *  @since 0.9.4
      */
-    void coalesceOnly() {
+    void coalesceOnly(boolean shouldDecay) {
     	_coalescing = true;
     	
     	//_receiveSize.coalesceStats();
@@ -565,7 +613,7 @@ public class PeerProfile {
     		_dbHistory.coalesceStats();
     	}
     	
-    	coalesceThroughput();
+    	coalesceThroughput(shouldDecay);
     	
     	_speedValueNew = calculateSpeed();
     	_capacityValueNew = calculateCapacity();
@@ -574,7 +622,6 @@ public class PeerProfile {
         // (in fact aren't really used at all), so we can
         // update them directly
     	_integrationValue = calculateIntegration();
-    	_isFailing = calculateIsFailing();
     }
     
     /**
@@ -583,7 +630,7 @@ public class PeerProfile {
      */
     void updateValues() {
     	if (!_coalescing) // can happen
-    		coalesceOnly();
+    		coalesceOnly(false);
     	_coalescing = false;
     	
     	_speedValue = _speedValueNew;
@@ -594,9 +641,7 @@ public class PeerProfile {
     private float calculateCapacity() { return (float) CapacityCalculator.calc(this); }
     private float calculateIntegration() { return (float) IntegrationCalculator.calc(this); }
     /** deprecated - unused - always false */
-    private boolean calculateIsFailing() { return false; }
-    /** deprecated - unused - always false */
-    void setIsFailing(boolean val) { _isFailing = val; }
+    void setIsFailing(boolean val) {}
     
     /**
      *  Helper for calculators
