@@ -33,6 +33,7 @@ import net.i2p.router.RouterContext;
 import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
 import net.i2p.router.transport.crypto.DHSessionKeyBuilder;
 import static net.i2p.router.transport.ntcp.OutboundNTCP2State.*;
+import net.i2p.util.ByteArrayStream;
 import net.i2p.util.ByteCache;
 import net.i2p.util.Log;
 import net.i2p.util.SimpleByteCache;
@@ -456,7 +457,7 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
             long rtt = now - _con.getCreated();
             _peerSkew = (now - (tsA * 1000) - (rtt / 2) + 500) / 1000; 
 
-            ByteArrayOutputStream baos = new ByteArrayOutputStream(768);
+            ByteArrayStream baos = new ByteArrayStream(256 + 256 + 32 + 4 + 4);
             baos.write(_X);
             baos.write(_Y);
             baos.write(_context.routerHash().getData());
@@ -672,7 +673,6 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
             }
             changeState(State.IB_NTCP2_GOT_X);
             _received = 0;
-
             // replay check using encrypted key
             if (!_transport.isHXHIValid(_X)) {
                 _context.statManager().addRateData("ntcp.replayHXxorBIH", 1);
@@ -680,13 +680,6 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
                 return;
             }
 
-            try {
-                _handshakeState = new HandshakeState(HandshakeState.PATTERN_ID_XK, HandshakeState.RESPONDER, _transport.getXDHFactory());
-            } catch (GeneralSecurityException gse) {
-                throw new IllegalStateException("bad proto", gse);
-            }
-            _handshakeState.getLocalKeyPair().setPublicKey(_transport.getNTCP2StaticPubkey(), 0);
-            _handshakeState.getLocalKeyPair().setPrivateKey(_transport.getNTCP2StaticPrivkey(), 0);
             Hash h = _context.routerHash();
             SessionKey bobHash = new SessionKey(h.getData());
             // save encrypted data for CBC for msg 2
@@ -696,6 +689,19 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
                 fail("Bad msg 1, X = 0");
                 return;
             }
+            // fast MSB check for key < 2^255
+            if ((_X[KEY_SIZE - 1] & 0x80) != 0) {
+                fail("Bad PK msg 1");
+                return;
+            }
+
+            try {
+                _handshakeState = new HandshakeState(HandshakeState.PATTERN_ID_XK, HandshakeState.RESPONDER, _transport.getXDHFactory());
+            } catch (GeneralSecurityException gse) {
+                throw new IllegalStateException("bad proto", gse);
+            }
+            _handshakeState.getLocalKeyPair().setKeys(_transport.getNTCP2StaticPrivkey(), 0,
+                                                      _transport.getNTCP2StaticPubkey(), 0);
             byte options[] = new byte[OPTIONS1_SIZE];
             try {
                 _handshakeState.start();
