@@ -253,8 +253,10 @@ class EstablishmentManager {
         Hash toHash = toIdentity.calculateHash();
         if (toRouterInfo.getNetworkId() != _networkID) {
             _context.banlist().banlistRouterForever(toHash, "Not in our network: " + toRouterInfo.getNetworkId());
+            if (_log.shouldWarn())
+                _log.warn("Not in our network: " + toRouterInfo, new Exception());
             _transport.markUnreachable(toHash);
-            _transport.failed(msg, "Remote peer is on the wrong network, cannot establish");
+            _transport.failed(msg, "Not in our network");
             return;
         }
         UDPAddress addr = new UDPAddress(ra);
@@ -941,7 +943,7 @@ class EstablishmentManager {
             state.setIntroNonce(nonce);
         }
         _context.statManager().addRateData("udp.sendIntroRelayRequest", 1);
-        List<UDPPacket> requests = _builder.buildRelayRequest(_transport, state, _transport.getIntroKey());
+        List<UDPPacket> requests = _builder.buildRelayRequest(_transport, this, state, _transport.getIntroKey());
         if (requests.isEmpty()) {
             // FIXME need a failed OB state
             if (_log.shouldLog(Log.WARN))
@@ -1045,9 +1047,9 @@ class EstablishmentManager {
      *  Are IP and port valid? This is only for checking the relay response.
      *  Reject all IPv6, for now, even if we are configured for it.
      *  Refuse anybody in the same /16
-     *  @since 0.9.3
+     *  @since 0.9.3, pkg private since 0.9.45 for PacketBuider
      */
-    private boolean isValid(byte[] ip, int port) {
+    boolean isValid(byte[] ip, int port) {
         return TransportUtil.isValidPort(port) &&
                ip != null && ip.length == 4 &&
                _transport.isValid(ip) &&
