@@ -74,6 +74,7 @@ class OutboundEstablishState {
     private long _confirmedSentTime;
     private long _requestSentTime;
     private long _introSentTime;
+    private int _rtt;
     
     public enum OutboundState {
         /** nothin sent yet */
@@ -179,6 +180,8 @@ class OutboundEstablishState {
      *  @since 0.9.24
      */
     public boolean needIntroduction() { return _needIntroduction; }
+
+    synchronized int getRTT() { return _rtt; }
     
     /**
      *  Queue a message to be sent after the session is established.
@@ -304,6 +307,10 @@ class OutboundEstablishState {
             _currentState == OutboundState.OB_STATE_INTRODUCED ||
             _currentState == OutboundState.OB_STATE_PENDING_INTRO)
             _currentState = OutboundState.OB_STATE_CREATED_RECEIVED;
+
+        if (_requestSentCount == 1) {
+            _rtt = (int) (_context.clock().now() - _requestSentTime);
+        }
         packetReceived();
     }
     
@@ -653,6 +660,12 @@ class OutboundEstablishState {
     /** how long have we been trying to establish this session? */
     public long getLifetime() { return _context.clock().now() - _establishBegin; }
     public long getEstablishBeginTime() { return _establishBegin; }
+
+    /**
+     *  @return 0 at initialization (to force sending session request),
+     *          rcv time after receiving a packet,
+     *          send time + delay after sending a packet (including session request)
+     */
     public synchronized long getNextSendTime() { return _nextSend; }
 
     /**
@@ -678,10 +691,13 @@ class OutboundEstablishState {
         _currentState = OutboundState.OB_STATE_CONFIRMED_COMPLETELY;
     }
     
+    /**
+     *  Call from synchronized method only
+     */
     private void packetReceived() {
         _nextSend = _context.clock().now();
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Got a packet, nextSend == now");
+        //if (_log.shouldLog(Log.DEBUG))
+        //    _log.debug("Got a packet, nextSend == now");
     }
 
     /** @since 0.8.9 */
