@@ -7,6 +7,7 @@ import net.i2p.data.Base64;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
 import net.i2p.data.SessionKey;
+import net.i2p.data.i2np.BuildResponseRecord;
 import net.i2p.data.i2np.EncryptedBuildRecord;
 import net.i2p.data.i2np.TunnelBuildReplyMessage;
 import net.i2p.util.Log;
@@ -50,19 +51,34 @@ public class BuildReplyHandler {
         for (int i = 0; i < rv.length; i++) {
             int hop = recordOrder.get(i).intValue();
             if (BuildMessageGenerator.isBlank(cfg, hop)) {
-                // self...
+                // self or unused...
                 if (log.shouldLog(Log.DEBUG))
-                    log.debug(reply.getUniqueId() + ": no need to decrypt record " + i + "/" + hop + ", as its out of range: " + cfg);
-                rv[i] = 0;
+                    log.debug(reply.getUniqueId() + ": skipping record " + i + "/" + hop + " for: " + cfg);
+                if (cfg.isInbound() && hop + 1 == cfg.getLength()) { // IBEP
+                    byte[] h1 = new byte[Hash.HASH_LENGTH];
+                    ctx.sha().calculateHash(reply.getRecord(i).getData(), 0, TunnelBuildReplyMessage.RECORD_SIZE, h1, 0);
+                    // get stored hash put here by BuildMessageGenerator
+                    Hash h2 = cfg.getBlankHash();
+                    if (h2 != null && DataHelper.eq(h1, h2.getData())) {
+                        rv[i] = 0;
+                    } else {
+                        if (log.shouldWarn())
+                            log.warn("IBEP record corrupt on " + cfg);
+                        // Caller doesn't check value for this hop so fail the whole thing
+                        return null;
+                    }
+                } else {
+                    rv[i] = 0;
+                }
             } else {
                 int ok = decryptRecord(reply, cfg, i, hop);
                 if (ok == -1) {
                     if (log.shouldLog(Log.WARN))
-                        log.warn(reply.getUniqueId() + ": decrypt record " + i + "/" + hop + " was not ok: " + cfg);
+                        log.warn(reply.getUniqueId() + ": decrypt record " + i + "/" + hop + " fail: " + cfg);
                     return null;
                 } else {
                     if (log.shouldLog(Log.DEBUG))
-                        log.debug(reply.getUniqueId() + ": decrypt record " + i + "/" + hop + " was ok: " + ok + " for " + cfg);
+                        log.debug(reply.getUniqueId() + ": decrypt record " + i + "/" + hop + " success: " + ok + " for " + cfg);
                 }
                 rv[i] = ok;
             }
@@ -75,25 +91,25 @@ public class BuildReplyHandler {
      *
      * Note that this layer-decrypts the build records in-place.
      * Do not call this more than once for a given message.
+     * Do not call for blank hops.
      *
      * @return the status 0-255, or -1 on decrypt failure
      */
     private int decryptRecord(TunnelBuildReplyMessage reply, TunnelCreatorConfig cfg, int recordNum, int hop) {
-        if (BuildMessageGenerator.isBlank(cfg, hop)) {
-            if (log.shouldLog(Log.DEBUG))
-                log.debug(reply.getUniqueId() + ": Record " + recordNum + "/" + hop + " is fake, so consider it valid...");
-            return 0;
-        }
         EncryptedBuildRecord rec = reply.getRecord(recordNum);
         byte[] data = rec.getData();
         int start = cfg.getLength() - 1;
         if (cfg.isInbound())
             start--; // the last hop in an inbound tunnel response doesn't actually encrypt
+        int end = hop;
+        boolean isEC = cfg.isEC(hop);
+        // chacha decrypt after the loop
+        if (isEC)
+            end++;
         // do we need to adjust this for the endpoint?
-        for (int j = start; j >= hop; j--) {
-            HopConfig hopConfig = cfg.getConfig(j);
-            SessionKey replyKey = hopConfig.getReplyKey();
-            byte replyIV[] = hopConfig.getReplyIV();
+        for (int j = start; j >= end; j--) {
+            SessionKey replyKey = cfg.getAESReplyKey(j);
+            byte replyIV[] = cfg.getAESReplyIV(j);
             if (log.shouldLog(Log.DEBUG)) {
                 log.debug(reply.getUniqueId() + ": Decrypting record " + recordNum + "/" + hop + "/" + j + " with replyKey " 
                           + replyKey.toBase64() + "/" + Base64.encode(replyIV) + ": " + cfg);
@@ -106,24 +122,41 @@ public class BuildReplyHandler {
         }
         // ok, all of the layered encryption is stripped, so lets verify it 
         // (formatted per BuildResponseRecord.create)
-        // don't cache the result
-        //Hash h = ctx.sha().calculateHash(data, off + Hash.HASH_LENGTH, TunnelBuildReplyMessage.RECORD_SIZE-Hash.HASH_LENGTH);
-        byte[] h = SimpleByteCache.acquire(Hash.HASH_LENGTH);
-        ctx.sha().calculateHash(data, Hash.HASH_LENGTH, TunnelBuildReplyMessage.RECORD_SIZE-Hash.HASH_LENGTH, h, 0);
-        boolean ok = DataHelper.eq(h, 0, data, 0, Hash.HASH_LENGTH);
-        if (!ok) {
-            if (log.shouldLog(Log.DEBUG))
-                log.debug(reply.getUniqueId() + ": Failed verification on " + recordNum + "/" + hop + ": " + Base64.encode(h) + " calculated, " +
-                          Base64.encode(data, 0, Hash.HASH_LENGTH) + " expected\n" +
-                          "Record: " + Base64.encode(data, Hash.HASH_LENGTH, TunnelBuildReplyMessage.RECORD_SIZE-Hash.HASH_LENGTH));
-            SimpleByteCache.release(h);
-            return -1;
+        int rv;
+        if (isEC) {
+            // For last iteration, do ChaCha instead
+            SessionKey replyKey = cfg.getChaChaReplyKey(hop);
+            byte[] replyIV = cfg.getChaChaReplyAD(hop);
+            if (log.shouldDebug())
+                log.debug(reply.getUniqueId() + ": Decrypting chacha/poly record " + recordNum + "/" + hop + " with replyKey " 
+                          + replyKey.toBase64() + "/" + Base64.encode(replyIV) + ": " + cfg);
+            boolean ok = BuildResponseRecord.decrypt(rec, replyKey, replyIV);
+            if (!ok) {
+                if (log.shouldWarn())
+                    log.debug(reply.getUniqueId() + ": chacha reply decrypt fail on " + recordNum + "/" + hop);
+                return -1;
+            }
+            // reply properties TODO
+            rv = data[TunnelBuildReplyMessage.RECORD_SIZE - 17] & 0xff;
         } else {
+            // don't cache the result
+            //Hash h = ctx.sha().calculateHash(data, off + Hash.HASH_LENGTH, TunnelBuildReplyMessage.RECORD_SIZE-Hash.HASH_LENGTH);
+            byte[] h = SimpleByteCache.acquire(Hash.HASH_LENGTH);
+            ctx.sha().calculateHash(data, Hash.HASH_LENGTH, TunnelBuildReplyMessage.RECORD_SIZE-Hash.HASH_LENGTH, h, 0);
+            boolean ok = DataHelper.eq(h, 0, data, 0, Hash.HASH_LENGTH);
+            if (!ok) {
+                if (log.shouldWarn())
+                    log.warn(reply.getUniqueId() + ": sha256 reply verify fail on " + recordNum + "/" + hop + ": " + Base64.encode(h) + " calculated, " +
+                             Base64.encode(data, 0, Hash.HASH_LENGTH) + " expected\n" +
+                             "Record: " + Base64.encode(data, Hash.HASH_LENGTH, TunnelBuildReplyMessage.RECORD_SIZE-Hash.HASH_LENGTH));
+                SimpleByteCache.release(h);
+                return -1;
+            }
             SimpleByteCache.release(h);
-            int rv = data[TunnelBuildReplyMessage.RECORD_SIZE - 1] & 0xff;
-            if (log.shouldLog(Log.DEBUG))
-                log.debug(reply.getUniqueId() + ": Verified: " + rv + " for record " + recordNum + "/" + hop);
-            return rv;
+            rv = data[TunnelBuildReplyMessage.RECORD_SIZE - 1] & 0xff;
         }
+        if (log.shouldLog(Log.DEBUG))
+            log.debug(reply.getUniqueId() + ": Verified: " + rv + " for record " + recordNum + "/" + hop);
+        return rv;
     }
 }
